@@ -22,6 +22,8 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
+//go:generate mockgen -copyright_file ../LICENSE -package $GOPACKAGE -source $GOFILE -destination clientFactory_mock.go
+
 package client
 
 import (
@@ -40,10 +42,11 @@ import (
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/membership"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 )
 
 const (
-	clientKeyConnection = "client-key-connection"
+	constClientKey = "const-client-key"
 )
 
 type (
@@ -55,8 +58,8 @@ type (
 
 		NewHistoryClientWithTimeout(timeout time.Duration) (historyservice.HistoryServiceClient, error)
 		NewMatchingClientWithTimeout(namespaceIDToName NamespaceIDToNameFunc, timeout time.Duration, longPollTimeout time.Duration) (matchingservice.MatchingServiceClient, error)
-		NewFrontendClientWithTimeout(rpcAddress string, timeout time.Duration, longPollTimeout time.Duration) (workflowservice.WorkflowServiceClient, error)
-		NewAdminClientWithTimeout(rpcAddress string, timeout time.Duration, largeTimeout time.Duration) (adminservice.AdminServiceClient, error)
+		NewFrontendClientWithTimeout(rpcAddress string, timeout time.Duration, longPollTimeout time.Duration) workflowservice.WorkflowServiceClient
+		NewAdminClientWithTimeout(rpcAddress string, timeout time.Duration, largeTimeout time.Duration) adminservice.AdminServiceClient
 	}
 
 	// FactoryProvider can be used to provide a customized client Factory implementation.
@@ -72,7 +75,7 @@ type (
 	}
 
 	// NamespaceIDToNameFunc maps a namespaceID to namespace name. Returns error when mapping is not possible.
-	NamespaceIDToNameFunc func(string) (string, error)
+	NamespaceIDToNameFunc func(id namespace.ID) (namespace.Name, error)
 
 	rpcClientFactory struct {
 		rpcFactory            common.RPCFactory
@@ -85,6 +88,13 @@ type (
 
 	factoryProviderImpl struct {
 	}
+
+	serviceKeyResolverImpl struct {
+		resolver membership.ServiceResolver
+	}
+
+	constKeyResolverImpl struct {
+	}
 )
 
 // NewFactoryProvider creates a default implementation of FactoryProvider.
@@ -120,7 +130,7 @@ func (cf *rpcClientFactory) NewMatchingClient(namespaceIDToName NamespaceIDToNam
 }
 
 func (cf *rpcClientFactory) NewFrontendClient(rpcAddress string) (workflowservice.WorkflowServiceClient, error) {
-	return cf.NewFrontendClientWithTimeout(rpcAddress, frontend.DefaultTimeout, frontend.DefaultLongPollTimeout)
+	return cf.NewFrontendClientWithTimeout(rpcAddress, frontend.DefaultTimeout, frontend.DefaultLongPollTimeout), nil
 }
 
 func (cf *rpcClientFactory) NewHistoryClientWithTimeout(timeout time.Duration) (historyservice.HistoryServiceClient, error) {
@@ -129,20 +139,13 @@ func (cf *rpcClientFactory) NewHistoryClientWithTimeout(timeout time.Duration) (
 		return nil, err
 	}
 
-	keyResolver := func(key string) (string, error) {
-		host, err := resolver.Lookup(key)
-		if err != nil {
-			return "", err
-		}
-		return host.GetAddress(), nil
-	}
-
+	keyResolver := newServiceKeyResolver(resolver)
 	clientProvider := func(clientKey string) (interface{}, error) {
 		connection := cf.rpcFactory.CreateInternodeGRPCConnection(clientKey)
 		return historyservice.NewHistoryServiceClient(connection), nil
 	}
-
-	client := history.NewClient(cf.numberOfHistoryShards, timeout, common.NewClientCache(keyResolver, clientProvider), cf.logger)
+	clientCache := common.NewClientCache(keyResolver, clientProvider)
+	client := history.NewClient(cf.numberOfHistoryShards, timeout, clientCache, cf.logger)
 	if cf.metricsClient != nil {
 		client = history.NewMetricClient(client, cf.metricsClient)
 	}
@@ -159,19 +162,11 @@ func (cf *rpcClientFactory) NewMatchingClientWithTimeout(
 		return nil, err
 	}
 
-	keyResolver := func(key string) (string, error) {
-		host, err := resolver.Lookup(key)
-		if err != nil {
-			return "", err
-		}
-		return host.GetAddress(), nil
-	}
-
+	keyResolver := newServiceKeyResolver(resolver)
 	clientProvider := func(clientKey string) (interface{}, error) {
 		connection := cf.rpcFactory.CreateInternodeGRPCConnection(clientKey)
 		return matchingservice.NewMatchingServiceClient(connection), nil
 	}
-
 	client := matching.NewClient(
 		timeout,
 		longPollTimeout,
@@ -190,32 +185,25 @@ func (cf *rpcClientFactory) NewFrontendClientWithTimeout(
 	rpcAddress string,
 	timeout time.Duration,
 	longPollTimeout time.Duration,
-) (workflowservice.WorkflowServiceClient, error) {
-	keyResolver := func(key string) (string, error) {
-		return clientKeyConnection, nil
-	}
-
+) workflowservice.WorkflowServiceClient {
+	keyResolver := newConstKeyResolver()
 	clientProvider := func(clientKey string) (interface{}, error) {
 		connection := cf.rpcFactory.CreateFrontendGRPCConnection(rpcAddress)
 		return workflowservice.NewWorkflowServiceClient(connection), nil
 	}
-
 	client := frontend.NewClient(timeout, longPollTimeout, common.NewClientCache(keyResolver, clientProvider))
 	if cf.metricsClient != nil {
 		client = frontend.NewMetricClient(client, cf.metricsClient)
 	}
-	return client, nil
+	return client
 }
 
 func (cf *rpcClientFactory) NewAdminClientWithTimeout(
 	rpcAddress string,
 	timeout time.Duration,
 	largeTimeout time.Duration,
-) (adminservice.AdminServiceClient, error) {
-	keyResolver := func(key string) (string, error) {
-		return clientKeyConnection, nil
-	}
-
+) adminservice.AdminServiceClient {
+	keyResolver := newConstKeyResolver()
 	clientProvider := func(clientKey string) (interface{}, error) {
 		connection := cf.rpcFactory.CreateFrontendGRPCConnection(rpcAddress)
 		return adminservice.NewAdminServiceClient(connection), nil
@@ -225,5 +213,41 @@ func (cf *rpcClientFactory) NewAdminClientWithTimeout(
 	if cf.metricsClient != nil {
 		client = admin.NewMetricClient(client, cf.metricsClient)
 	}
-	return client, nil
+	return client
+}
+
+func newServiceKeyResolver(resolver membership.ServiceResolver) *serviceKeyResolverImpl {
+	return &serviceKeyResolverImpl{
+		resolver: resolver,
+	}
+}
+
+func newConstKeyResolver() *constKeyResolverImpl {
+	return &constKeyResolverImpl{}
+}
+
+func (r *serviceKeyResolverImpl) Lookup(key string) (string, error) {
+	host, err := r.resolver.Lookup(key)
+	if err != nil {
+		return "", err
+	}
+	return host.GetAddress(), nil
+}
+
+func (r *serviceKeyResolverImpl) GetAllAddresses() ([]string, error) {
+	var all []string
+
+	for _, host := range r.resolver.Members() {
+		all = append(all, host.GetAddress())
+	}
+
+	return all, nil
+}
+
+func (r *constKeyResolverImpl) Lookup(_ string) (string, error) {
+	return constClientKey, nil
+}
+
+func (r *constKeyResolverImpl) GetAllAddresses() ([]string, error) {
+	return []string{constClientKey}, nil
 }
