@@ -92,6 +92,15 @@ func (c *metricClient) CloseShard(
 	return resp, err
 }
 
+func (c *metricClient) GetShard(
+	context context.Context,
+	request *historyservice.GetShardRequest,
+	opts ...grpc.CallOption) (*historyservice.GetShardResponse, error) {
+	resp, err := c.client.GetShard(context, request, opts...)
+
+	return resp, err
+}
+
 func (c *metricClient) DescribeMutableState(
 	context context.Context,
 	request *historyservice.DescribeMutableStateRequest,
@@ -626,3 +635,35 @@ func (c *metricClient) RefreshWorkflowTasks(
 	}
 	return resp, err
 }
+
+func (c *metricClient) GenerateLastHistoryReplicationTasks(
+	ctx context.Context,
+	request *historyservice.GenerateLastHistoryReplicationTasksRequest,
+	opts ...grpc.CallOption,
+) (*historyservice.GenerateLastHistoryReplicationTasksResponse, error) {
+	c.metricsClient.IncCounter(metrics.HistoryClientGenerateLastHistoryReplicationTasksScope, metrics.ClientRequests)
+	sw := c.metricsClient.StartTimer(metrics.HistoryClientGenerateLastHistoryReplicationTasksScope, metrics.ClientLatency)
+	resp, err := c.client.GenerateLastHistoryReplicationTasks(ctx, request, opts...)
+	sw.Stop()
+
+	if err != nil {
+		c.metricsClient.IncCounter(metrics.HistoryClientGenerateLastHistoryReplicationTasksScope, metrics.ClientFailures)
+	}
+	return resp, err
+}
+
+func (c *metricClient) GetReplicationStatus(
+	ctx context.Context,
+	request *historyservice.GetReplicationStatusRequest,
+	opts ...grpc.CallOption,
+) (*historyservice.GetReplicationStatusResponse, error) {
+	c.metricsClient.IncCounter(metrics.HistoryClientGetReplicationStatusScope, metrics.ClientRequests)
+	sw := c.metricsClient.StartTimer(metrics.HistoryClientGetReplicationStatusScope, metrics.ClientLatency)
+	resp, err := c.client.GetReplicationStatus(ctx, request, opts...)
+	sw.Stop()
+
+	if err != nil {
+		c.metricsClient.IncCounter(metrics.HistoryClientGetReplicationStatusScope, metrics.ClientFailures)
+	}
+	return resp, err
+}
