@@ -98,6 +98,22 @@ func (c *retryableClient) CloseShard(
 	return resp, err
 }
 
+func (c *retryableClient) GetShard(
+	ctx context.Context,
+	request *historyservice.GetShardRequest,
+	opts ...grpc.CallOption) (*historyservice.GetShardResponse, error) {
+
+	var resp *historyservice.GetShardResponse
+	op := func() error {
+		var err error
+		resp, err = c.client.GetShard(ctx, request, opts...)
+		return err
+	}
+
+	err := backoff.Retry(op, c.policy, c.isRetryable)
+	return resp, err
+}
+
 func (c *retryableClient) RemoveTask(
 	ctx context.Context,
 	request *historyservice.RemoveTaskRequest,
@@ -626,3 +642,37 @@ func (c *retryableClient) RefreshWorkflowTasks(
 	err := backoff.Retry(op, c.policy, c.isRetryable)
 	return resp, err
 }
+
+func (c *retryableClient) GenerateLastHistoryReplicationTasks(
+	ctx context.Context,
+	request *historyservice.GenerateLastHistoryReplicationTasksRequest,
+	opts ...grpc.CallOption,
+) (*historyservice.GenerateLastHistoryReplicationTasksResponse, error) {
+
+	var resp *historyservice.GenerateLastHistoryReplicationTasksResponse
+	op := func() error {
+		var err error
+		resp, err = c.client.GenerateLastHistoryReplicationTasks(ctx, request, opts...)
+		return err
+	}
+
+	err := backoff.Retry(op, c.policy, c.isRetryable)
+	return resp, err
+}
+
+func (c *retryableClient) GetReplicationStatus(
+	ctx context.Context,
+	request *historyservice.GetReplicationStatusRequest,
+	opts ...grpc.CallOption,
+) (*historyservice.GetReplicationStatusResponse, error) {
+
+	var resp *historyservice.GetReplicationStatusResponse
+	op := func() error {
+		var err error
+		resp, err = c.client.GetReplicationStatus(ctx, request, opts...)
+		return err
+	}
+
+	err := backoff.Retry(op, c.policy, c.isRetryable)
+	return resp, err
+}
