@@ -64,13 +64,14 @@ var Keys = map[Key]string{
 	AdminMatchingNamespaceToPartitionDispatchRate:          "admin.matchingNamespaceToPartitionDispatchRate",
 	AdminMatchingNamespaceTaskqueueToPartitionDispatchRate: "admin.matchingNamespaceTaskqueueToPartitionDispatchRate",
 
-	// TODO remove this dynamic flag in 1.14.x
-	EnableDBRecordVersion: "system.enableDBRecordVersion",
-
 	// system settings
-	EnableVisibilitySampling:               "system.enableVisibilitySampling",
-	AdvancedVisibilityWritingMode:          "system.advancedVisibilityWritingMode",
-	EnableReadVisibilityFromES:             "system.enableReadVisibilityFromES",
+	StandardVisibilityPersistenceMaxReadQPS:  "system.standardVisibilityPersistenceMaxReadQPS",
+	StandardVisibilityPersistenceMaxWriteQPS: "system.standardVisibilityPersistenceMaxWriteQPS",
+	AdvancedVisibilityPersistenceMaxReadQPS:  "system.advancedVisibilityPersistenceMaxReadQPS",
+	AdvancedVisibilityPersistenceMaxWriteQPS: "system.advancedVisibilityPersistenceMaxWriteQPS",
+	AdvancedVisibilityWritingMode:            "system.advancedVisibilityWritingMode",
+	EnableReadVisibilityFromES:               "system.enableReadVisibilityFromES",
+
 	HistoryArchivalState:                   "system.historyArchivalState",
 	EnableReadFromHistoryArchival:          "system.enableReadFromHistoryArchival",
 	VisibilityArchivalState:                "system.visibilityArchivalState",
@@ -100,13 +101,12 @@ var Keys = map[Key]string{
 	FrontendPersistenceMaxQPS:             "frontend.persistenceMaxQPS",
 	FrontendPersistenceGlobalMaxQPS:       "frontend.persistenceGlobalMaxQPS",
 	FrontendVisibilityMaxPageSize:         "frontend.visibilityMaxPageSize",
-	FrontendVisibilityListMaxQPS:          "frontend.visibilityListMaxQPS",
-	FrontendESVisibilityListMaxQPS:        "frontend.esVisibilityListMaxQPS",
 	FrontendMaxBadBinaries:                "frontend.maxBadBinaries",
 	FrontendESIndexMaxResultWindow:        "frontend.esIndexMaxResultWindow",
 	FrontendHistoryMaxPageSize:            "frontend.historyMaxPageSize",
 	FrontendRPS:                           "frontend.rps",
 	FrontendMaxNamespaceRPSPerInstance:    "frontend.namespaceRPS",
+	FrontendMaxNamespaceBurstPerInstance:  "frontend.namespaceBurst",
 	FrontendMaxNamespaceCountPerInstance:  "frontend.namespaceCount",
 	FrontendGlobalNamespaceRPS:            "frontend.globalNamespacerps",
 	FrontendShutdownDrainDuration:         "frontend.shutdownDrainDuration",
@@ -151,15 +151,12 @@ var Keys = map[Key]string{
 	MatchingForwarderMaxOutstandingTasks:    "matching.forwarderMaxOutstandingTasks",
 	MatchingForwarderMaxRatePerSecond:       "matching.forwarderMaxRatePerSecond",
 	MatchingForwarderMaxChildrenPerNode:     "matching.forwarderMaxChildrenPerNode",
-	ResilientSyncMatch:                      "matching.resilientSyncMatch",
 	MatchingShutdownDrainDuration:           "matching.shutdownDrainDuration",
 
 	// history settings
 	HistoryRPS:                                           "history.rps",
 	HistoryPersistenceMaxQPS:                             "history.persistenceMaxQPS",
 	HistoryPersistenceGlobalMaxQPS:                       "history.persistenceGlobalMaxQPS",
-	HistoryVisibilityOpenMaxQPS:                          "history.historyVisibilityOpenMaxQPS",
-	HistoryVisibilityClosedMaxQPS:                        "history.historyVisibilityClosedMaxQPS",
 	HistoryLongPollExpirationInterval:                    "history.longPollExpirationInterval",
 	HistoryCacheInitialSize:                              "history.cacheInitialSize",
 	HistoryMaxAutoResetPoints:                            "history.historyMaxAutoResetPoints",
@@ -235,6 +232,24 @@ var Keys = map[Key]string{
 	VisibilityProcessorEnablePriorityTaskProcessor:         "history.visibilityProcessorEnablePriorityTaskProcessor",
 	VisibilityProcessorVisibilityArchivalTimeLimit:         "history.visibilityProcessorVisibilityArchivalTimeLimit",
 
+	TieredStorageTaskBatchSize:                                "history.tieredStorageTaskBatchSize",
+	TieredStorageProcessorFailoverMaxPollRPS:                  "history.tieredStorageProcessorFailoverMaxPollRPS",
+	TieredStorageProcessorMaxPollRPS:                          "history.tieredStorageProcessorMaxPollRPS",
+	TieredStorageTaskWorkerCount:                              "history.tieredStorageTaskWorkerCount",
+	TieredStorageTaskMaxRetryCount:                            "history.tieredStorageTaskMaxRetryCount",
+	TieredStorageProcessorCompleteTaskFailureRetryCount:       "history.tieredStorageProcessorCompleteTaskFailureRetryCount",
+	TieredStorageProcessorUpdateShardTaskCount:                "history.tieredStorageProcessorUpdateShardTaskCount",
+	TieredStorageProcessorMaxPollInterval:                     "history.tieredStorageProcessorMaxPollInterval",
+	TieredStorageProcessorMaxPollIntervalJitterCoefficient:    "history.tieredStorageProcessorMaxPollIntervalJitterCoefficient",
+	TieredStorageProcessorUpdateAckInterval:                   "history.tieredStorageProcessorUpdateAckInterval",
+	TieredStorageProcessorUpdateAckIntervalJitterCoefficient:  "history.tieredStorageProcessorUpdateAckIntervalJitterCoefficient",
+	TieredStorageProcessorCompleteTaskInterval:                "history.tieredStorageProcessorCompleteTaskInterval",
+	TieredStorageProcessorRedispatchInterval:                  "history.tieredStorageProcessorRedispatchInterval",
+	TieredStorageProcessorRedispatchIntervalJitterCoefficient: "history.tieredStorageProcessorRedispatchIntervalJitterCoefficient",
+	TieredStorageProcessorMaxRedispatchQueueSize:              "history.tieredStorageProcessorMaxRedispatchQueueSize",
+	TieredStorageProcessorEnablePriorityTaskProcessor:         "history.tieredStorageProcessorEnablePriorityTaskProcessor",
+	TieredStorageProcessorArchivalTimeLimit:                   "history.tieredStorageProcessorArchivalTimeLimit",
+
 	ReplicatorTaskBatchSize:                                "history.replicatorTaskBatchSize",
 	ReplicatorTaskWorkerCount:                              "history.replicatorTaskWorkerCount",
 	ReplicatorTaskMaxRetryCount:                            "history.replicatorTaskMaxRetryCount",
@@ -330,6 +345,8 @@ var Keys = map[Key]string{
 	TaskQueueScannerEnabled:             "worker.taskQueueScannerEnabled",
 	HistoryScannerEnabled:               "worker.historyScannerEnabled",
 	ExecutionsScannerEnabled:            "worker.executionsScannerEnabled",
+
+	EnableRingpopTLS: "system.enableRingpopTLS",
 }
 
 const (
@@ -356,21 +373,23 @@ const (
 	// AdminMatchingNamespaceTaskqueueToPartitionDispatchRate is the max qps of a task queue partition for a given namespace & task queue
 	AdminMatchingNamespaceTaskqueueToPartitionDispatchRate
 
-	// TODO remove this dynamic flag in 1.14.x
-
-	// EnableDBRecordVersion is key for enable db version
-	EnableDBRecordVersion
-
-	// EnableVisibilitySampling is key for enable visibility sampling
-	EnableVisibilitySampling
+	// StandardVisibilityPersistenceMaxReadQPS is the max QPC system host can query standard visibility DB (SQL or Cassandra) for read.
+	StandardVisibilityPersistenceMaxReadQPS
+	// StandardVisibilityPersistenceMaxWriteQPS is the max QPC system host can query standard visibility DB (SQL or Cassandra) for write.
+	StandardVisibilityPersistenceMaxWriteQPS
+	// AdvancedVisibilityPersistenceMaxReadQPS is the max QPC system host can query advanced visibility DB (Elasticsearch) for read.
+	AdvancedVisibilityPersistenceMaxReadQPS
+	// AdvancedVisibilityPersistenceMaxWriteQPS is the max QPC system host can query advanced visibility DB (Elasticsearch) for write.
+	AdvancedVisibilityPersistenceMaxWriteQPS
 	// AdvancedVisibilityWritingMode is key for how to write to advanced visibility
 	AdvancedVisibilityWritingMode
-	// EmitShardDiffLog whether emit the shard diff log
-	EmitShardDiffLog
 	// EnableReadVisibilityFromES is key for enable read from elastic search
 	EnableReadVisibilityFromES
 	// DisableListVisibilityByFilter is config to disable list open/close workflow using filter
 	DisableListVisibilityByFilter
+
+	// EmitShardDiffLog whether emit the shard diff log
+	EmitShardDiffLog
 	// HistoryArchivalState is key for the state of history archival
 	HistoryArchivalState
 	// EnableReadFromHistoryArchival is key for enabling reading history from archival store
@@ -419,20 +438,20 @@ const (
 	FrontendPersistenceMaxQPS
 	// FrontendPersistenceGlobalMaxQPS is the max qps frontend cluster can query DB
 	FrontendPersistenceGlobalMaxQPS
+
 	// FrontendVisibilityMaxPageSize is default max size for ListWorkflowExecutions in one page
 	FrontendVisibilityMaxPageSize
-	// FrontendVisibilityListMaxQPS is max qps frontend can list open/close workflows
-	FrontendVisibilityListMaxQPS
-	// FrontendESVisibilityListMaxQPS is max qps frontend can list open/close workflows from ElasticSearch
-	FrontendESVisibilityListMaxQPS
 	// FrontendESIndexMaxResultWindow is ElasticSearch index setting max_result_window
 	FrontendESIndexMaxResultWindow
+
 	// FrontendHistoryMaxPageSize is default max size for GetWorkflowExecutionHistory in one page
 	FrontendHistoryMaxPageSize
 	// FrontendRPS is workflow rate limit per second
 	FrontendRPS
 	// FrontendMaxNamespaceRPSPerInstance is workflow namespace rate limit per second
 	FrontendMaxNamespaceRPSPerInstance
+	// FrontendMaxNamespaceBurstPerInstance is workflow namespace burst limit
+	FrontendMaxNamespaceBurstPerInstance
 	// FrontendMaxNamespaceCountPerInstance is workflow namespace count limit per second
 	FrontendMaxNamespaceCountPerInstance
 	// FrontendGlobalNamespaceRPS is workflow namespace rate limit per second for the whole cluster
@@ -533,8 +552,6 @@ const (
 	MatchingForwarderMaxRatePerSecond
 	// MatchingForwarderMaxChildrenPerNode is the max number of children per node in the task queue partition tree
 	MatchingForwarderMaxChildrenPerNode
-	// ResilientSyncMatch enables or disables sync-matching while queue persistence is unavailable
-	ResilientSyncMatch
 	// MatchingShutdownDrainDuration is the duration of traffic drain during shutdown
 	MatchingShutdownDrainDuration
 
@@ -546,10 +563,6 @@ const (
 	HistoryPersistenceMaxQPS
 	// HistoryPersistenceGlobalMaxQPS is the max qps history cluster can query DB
 	HistoryPersistenceGlobalMaxQPS
-	// HistoryVisibilityOpenMaxQPS is max qps one history host can write visibility open_executions
-	HistoryVisibilityOpenMaxQPS
-	// HistoryVisibilityClosedMaxQPS is max qps one history host can write visibility closed_executions
-	HistoryVisibilityClosedMaxQPS
 	// HistoryLongPollExpirationInterval is the long poll expiration interval in the history service
 	HistoryLongPollExpirationInterval
 	// HistoryCacheInitialSize is initial size of history cache
@@ -698,6 +711,41 @@ const (
 	// VisibilityProcessorVisibilityArchivalTimeLimit is the upper time limit for archiving visibility records
 	VisibilityProcessorVisibilityArchivalTimeLimit
 
+	// TieredStorageTaskBatchSize is batch size for TieredStorageQueueProcessor
+	TieredStorageTaskBatchSize
+	// TieredStorageProcessorFailoverMaxPollRPS is max poll rate per second for TieredStorageQueueProcessor
+	TieredStorageProcessorFailoverMaxPollRPS
+	// TieredStorageProcessorMaxPollRPS is max poll rate per second for TieredStorageQueueProcessor
+	TieredStorageProcessorMaxPollRPS
+	// TieredStorageTaskWorkerCount is number of worker for TieredStorageQueueProcessor
+	TieredStorageTaskWorkerCount
+	// TieredStorageTaskMaxRetryCount is max times of retry for TieredStorageQueueProcessor
+	TieredStorageTaskMaxRetryCount
+	// TieredStorageProcessorCompleteTaskFailureRetryCount is times of retry for failure
+	TieredStorageProcessorCompleteTaskFailureRetryCount
+	// TieredStorageProcessorUpdateShardTaskCount is update shard count for TieredStorageQueueProcessor
+	TieredStorageProcessorUpdateShardTaskCount
+	// TieredStorageProcessorMaxPollInterval max poll interval for TieredStorageQueueProcessor
+	TieredStorageProcessorMaxPollInterval
+	// TieredStorageProcessorMaxPollIntervalJitterCoefficient is the max poll interval jitter coefficient
+	TieredStorageProcessorMaxPollIntervalJitterCoefficient
+	// TieredStorageProcessorUpdateAckInterval is update interval for TieredStorageQueueProcessor
+	TieredStorageProcessorUpdateAckInterval
+	// TieredStorageProcessorUpdateAckIntervalJitterCoefficient is the update interval jitter coefficient
+	TieredStorageProcessorUpdateAckIntervalJitterCoefficient
+	// TieredStorageProcessorCompleteTaskInterval is complete timer interval for TieredStorageQueueProcessor
+	TieredStorageProcessorCompleteTaskInterval
+	// TieredStorageProcessorRedispatchInterval is the redispatch interval for TieredStorageQueueProcessor
+	TieredStorageProcessorRedispatchInterval
+	// TieredStorageProcessorRedispatchIntervalJitterCoefficient is the redispatch interval jitter coefficient
+	TieredStorageProcessorRedispatchIntervalJitterCoefficient
+	// TieredStorageProcessorMaxRedispatchQueueSize is the threshold of the number of tasks in the redispatch queue for TieredStorageQueueProcessor
+	TieredStorageProcessorMaxRedispatchQueueSize
+	// TieredStorageProcessorEnablePriorityTaskProcessor indicates whether priority task processor should be used for TieredStorageQueueProcessor
+	TieredStorageProcessorEnablePriorityTaskProcessor
+	// TieredStorageProcessorArchivalTimeLimit is the upper time limit for archiving TieredStorage records
+	TieredStorageProcessorArchivalTimeLimit
+
 	// ReplicatorTaskBatchSize is batch size for ReplicatorProcessor
 	ReplicatorTaskBatchSize
 	// ReplicatorTaskWorkerCount is number of worker for ReplicatorProcessor
@@ -893,6 +941,8 @@ const (
 	// StandbyTaskReReplicationContextTimeout is the context timeout for standby task re-replication
 	StandbyTaskReReplicationContextTimeout
 
+	EnableRingpopTLS
+
 	// lastKeyForTest must be the last one in this const group for testing purpose
 	lastKeyForTest
 )
