@@ -58,7 +58,6 @@ type (
 
 		controller *gomock.Controller
 
-		minRetentionDays        int
 		maxBadBinaryCount       int
 		metadataMgr             persistence.MetadataManager
 		mockProducer            *persistence.MockNamespaceReplicationQueue
@@ -87,7 +86,6 @@ func (s *namespaceHandlerGlobalNamespaceEnabledMasterClusterSuite) TearDownSuite
 func (s *namespaceHandlerGlobalNamespaceEnabledMasterClusterSuite) SetupTest() {
 	logger := log.NewNoopLogger()
 	dcCollection := dc.NewCollection(dc.NewNoopClient(), logger)
-	s.minRetentionDays = 1
 	s.maxBadBinaryCount = 10
 	s.metadataMgr = s.TestBase.MetadataManager
 	s.controller = gomock.NewController(s.T())
@@ -103,7 +101,6 @@ func (s *namespaceHandlerGlobalNamespaceEnabledMasterClusterSuite) SetupTest() {
 	)
 	s.mockArchiverProvider = provider.NewMockArchiverProvider(s.controller)
 	s.handler = NewHandler(
-		s.minRetentionDays,
 		dc.GetIntPropertyFilteredByNamespace(s.maxBadBinaryCount),
 		logger,
 		s.metadataMgr,
