@@ -34,40 +34,39 @@ import (
 )
 
 type (
-	shardPersistenceClient struct {
+	metricEmitter struct {
 		metricClient metrics.Client
-		persistence  ShardManager
 		logger       log.Logger
 	}
 
+	shardPersistenceClient struct {
+		metricEmitter
+		persistence ShardManager
+	}
+
 	executionPersistenceClient struct {
-		metricClient metrics.Client
-		persistence  ExecutionManager
-		logger       log.Logger
+		metricEmitter
+		persistence ExecutionManager
 	}
 
 	taskPersistenceClient struct {
-		metricClient metrics.Client
-		persistence  TaskManager
-		logger       log.Logger
+		metricEmitter
+		persistence TaskManager
 	}
 
 	metadataPersistenceClient struct {
-		metricClient metrics.Client
-		persistence  MetadataManager
-		logger       log.Logger
+		metricEmitter
+		persistence MetadataManager
 	}
 
 	clusterMetadataPersistenceClient struct {
-		metricClient metrics.Client
-		persistence  ClusterMetadataManager
-		logger       log.Logger
+		metricEmitter
+		persistence ClusterMetadataManager
 	}
 
 	queuePersistenceClient struct {
-		metricClient metrics.Client
-		persistence  Queue
-		logger       log.Logger
+		metricEmitter
+		persistence Queue
 	}
 )
 
@@ -81,54 +80,66 @@ var _ Queue = (*queuePersistenceClient)(nil)
 // NewShardPersistenceMetricsClient creates a client to manage shards
 func NewShardPersistenceMetricsClient(persistence ShardManager, metricClient metrics.Client, logger log.Logger) ShardManager {
 	return &shardPersistenceClient{
-		persistence:  persistence,
-		metricClient: metricClient,
-		logger:       logger,
+		metricEmitter: metricEmitter{
+			metricClient: metricClient,
+			logger:       logger,
+		},
+		persistence: persistence,
 	}
 }
 
 // NewExecutionPersistenceMetricsClient creates a client to manage executions
 func NewExecutionPersistenceMetricsClient(persistence ExecutionManager, metricClient metrics.Client, logger log.Logger) ExecutionManager {
 	return &executionPersistenceClient{
-		persistence:  persistence,
-		metricClient: metricClient,
-		logger:       logger,
+		metricEmitter: metricEmitter{
+			metricClient: metricClient,
+			logger:       logger,
+		},
+		persistence: persistence,
 	}
 }
 
 // NewTaskPersistenceMetricsClient creates a client to manage tasks
 func NewTaskPersistenceMetricsClient(persistence TaskManager, metricClient metrics.Client, logger log.Logger) TaskManager {
 	return &taskPersistenceClient{
-		persistence:  persistence,
-		metricClient: metricClient,
-		logger:       logger,
+		metricEmitter: metricEmitter{
+			metricClient: metricClient,
+			logger:       logger,
+		},
+		persistence: persistence,
 	}
 }
 
 // NewMetadataPersistenceMetricsClient creates a MetadataManager client to manage metadata
 func NewMetadataPersistenceMetricsClient(persistence MetadataManager, metricClient metrics.Client, logger log.Logger) MetadataManager {
 	return &metadataPersistenceClient{
-		persistence:  persistence,
-		metricClient: metricClient,
-		logger:       logger,
+		metricEmitter: metricEmitter{
+			metricClient: metricClient,
+			logger:       logger,
+		},
+		persistence: persistence,
 	}
 }
 
 // NewClusterMetadataPersistenceMetricsClient creates a ClusterMetadataManager client to manage cluster metadata
 func NewClusterMetadataPersistenceMetricsClient(persistence ClusterMetadataManager, metricClient metrics.Client, logger log.Logger) ClusterMetadataManager {
 	return &clusterMetadataPersistenceClient{
-		persistence:  persistence,
-		metricClient: metricClient,
-		logger:       logger,
+		metricEmitter: metricEmitter{
+			metricClient: metricClient,
+			logger:       logger,
+		},
+		persistence: persistence,
 	}
 }
 
 // NewQueuePersistenceMetricsClient creates a client to manage queue
 func NewQueuePersistenceMetricsClient(persistence Queue, metricClient metrics.Client, logger log.Logger) Queue {
 	return &queuePersistenceClient{
-		persistence:  persistence,
-		metricClient: metricClient,
-		logger:       logger,
+		metricEmitter: metricEmitter{
+			metricClient: metricClient,
+			logger:       logger,
+		},
+		persistence: persistence,
 	}
 }
 
@@ -136,30 +147,16 @@ func (p *shardPersistenceClient) GetName() string {
 	return p.persistence.GetName()
 }
 
-func (p *shardPersistenceClient) CreateShard(request *CreateShardRequest) error {
-	p.metricClient.IncCounter(metrics.PersistenceCreateShardScope, metrics.PersistenceRequests)
+func (p *shardPersistenceClient) GetOrCreateShard(
+	request *GetOrCreateShardRequest) (*GetOrCreateShardResponse, error) {
+	p.metricClient.IncCounter(metrics.PersistenceGetOrCreateShardScope, metrics.PersistenceRequests)
 
-	sw := p.metricClient.StartTimer(metrics.PersistenceCreateShardScope, metrics.PersistenceLatency)
-	err := p.persistence.CreateShard(request)
+	sw := p.metricClient.StartTimer(metrics.PersistenceGetOrCreateShardScope, metrics.PersistenceLatency)
+	response, err := p.persistence.GetOrCreateShard(request)
 	sw.Stop()
 
 	if err != nil {
-		p.updateErrorMetric(metrics.PersistenceCreateShardScope, err)
-	}
-
-	return err
-}
-
-func (p *shardPersistenceClient) GetShard(
-	request *GetShardRequest) (*GetShardResponse, error) {
-	p.metricClient.IncCounter(metrics.PersistenceGetShardScope, metrics.PersistenceRequests)
-
-	sw := p.metricClient.StartTimer(metrics.PersistenceGetShardScope, metrics.PersistenceLatency)
-	response, err := p.persistence.GetShard(request)
-	sw.Stop()
-
-	if err != nil {
-		p.updateErrorMetric(metrics.PersistenceGetShardScope, err)
+		p.updateErrorMetric(metrics.PersistenceGetOrCreateShardScope, err)
 	}
 
 	return response, err
@@ -179,23 +176,6 @@ func (p *shardPersistenceClient) UpdateShard(request *UpdateShardRequest) error
 	return err
 }
 
-func (p *shardPersistenceClient) updateErrorMetric(scope int, err error) {
-	switch err.(type) {
-	case *ShardAlreadyExistError:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrShardExistsCounter)
-	case *ShardOwnershipLostError:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrShardOwnershipLostCounter)
-	case *serviceerror.NotFound:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrEntityNotExistsCounter)
-	case *serviceerror.ResourceExhausted:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrBusyCounter)
-		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
-	default:
-		p.logger.Error("Operation failed with internal error.", tag.Error(err), tag.MetricScope(scope))
-		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
-	}
-}
-
 func (p *shardPersistenceClient) Close() {
 	p.persistence.Close()
 }
@@ -246,18 +226,18 @@ func (p *executionPersistenceClient) UpdateWorkflowExecution(request *UpdateWork
 	return resp, err
 }
 
-func (p *executionPersistenceClient) ConflictResolveWorkflowExecution(request *ConflictResolveWorkflowExecutionRequest) error {
+func (p *executionPersistenceClient) ConflictResolveWorkflowExecution(request *ConflictResolveWorkflowExecutionRequest) (*ConflictResolveWorkflowExecutionResponse, error) {
 	p.metricClient.IncCounter(metrics.PersistenceConflictResolveWorkflowExecutionScope, metrics.PersistenceRequests)
 
 	sw := p.metricClient.StartTimer(metrics.PersistenceConflictResolveWorkflowExecutionScope, metrics.PersistenceLatency)
-	err := p.persistence.ConflictResolveWorkflowExecution(request)
+	response, err := p.persistence.ConflictResolveWorkflowExecution(request)
 	sw.Stop()
 
 	if err != nil {
 		p.updateErrorMetric(metrics.PersistenceConflictResolveWorkflowExecutionScope, err)
 	}
 
-	return err
+	return response, err
 }
 
 func (p *executionPersistenceClient) DeleteWorkflowExecution(request *DeleteWorkflowExecutionRequest) error {
@@ -386,6 +366,34 @@ func (p *executionPersistenceClient) GetVisibilityTasks(request *GetVisibilityTa
 	return response, err
 }
 
+func (p *executionPersistenceClient) GetTieredStorageTask(request *GetTieredStorageTaskRequest) (*GetTieredStorageTaskResponse, error) {
+	p.metricClient.IncCounter(metrics.PersistenceGetTieredStorageTaskScope, metrics.PersistenceRequests)
+
+	sw := p.metricClient.StartTimer(metrics.PersistenceGetTieredStorageTaskScope, metrics.PersistenceLatency)
+	response, err := p.persistence.GetTieredStorageTask(request)
+	sw.Stop()
+
+	if err != nil {
+		p.updateErrorMetric(metrics.PersistenceGetTieredStorageTaskScope, err)
+	}
+
+	return response, err
+}
+
+func (p *executionPersistenceClient) GetTieredStorageTasks(request *GetTieredStorageTasksRequest) (*GetTieredStorageTasksResponse, error) {
+	p.metricClient.IncCounter(metrics.PersistenceGetTieredStorageTasksScope, metrics.PersistenceRequests)
+
+	sw := p.metricClient.StartTimer(metrics.PersistenceGetTieredStorageTasksScope, metrics.PersistenceLatency)
+	response, err := p.persistence.GetTieredStorageTasks(request)
+	sw.Stop()
+
+	if err != nil {
+		p.updateErrorMetric(metrics.PersistenceGetTieredStorageTasksScope, err)
+	}
+
+	return response, err
+}
+
 func (p *executionPersistenceClient) GetReplicationTask(request *GetReplicationTaskRequest) (*GetReplicationTaskResponse, error) {
 	p.metricClient.IncCounter(metrics.PersistenceGetReplicationTaskScope, metrics.PersistenceRequests)
 
@@ -470,6 +478,34 @@ func (p *executionPersistenceClient) RangeCompleteVisibilityTask(request *RangeC
 	return err
 }
 
+func (p *executionPersistenceClient) CompleteTieredStorageTask(request *CompleteTieredStorageTaskRequest) error {
+	p.metricClient.IncCounter(metrics.PersistenceCompleteTieredStorageTaskScope, metrics.PersistenceRequests)
+
+	sw := p.metricClient.StartTimer(metrics.PersistenceCompleteTieredStorageTaskScope, metrics.PersistenceLatency)
+	err := p.persistence.CompleteTieredStorageTask(request)
+	sw.Stop()
+
+	if err != nil {
+		p.updateErrorMetric(metrics.PersistenceCompleteTieredStorageTaskScope, err)
+	}
+
+	return err
+}
+
+func (p *executionPersistenceClient) RangeCompleteTieredStorageTask(request *RangeCompleteTieredStorageTaskRequest) error {
+	p.metricClient.IncCounter(metrics.PersistenceRangeCompleteTieredStorageTaskScope, metrics.PersistenceRequests)
+
+	sw := p.metricClient.StartTimer(metrics.PersistenceRangeCompleteTieredStorageTaskScope, metrics.PersistenceLatency)
+	err := p.persistence.RangeCompleteTieredStorageTask(request)
+	sw.Stop()
+
+	if err != nil {
+		p.updateErrorMetric(metrics.PersistenceRangeCompleteTieredStorageTaskScope, err)
+	}
+
+	return err
+}
+
 func (p *executionPersistenceClient) CompleteReplicationTask(request *CompleteReplicationTaskRequest) error {
 	p.metricClient.IncCounter(metrics.PersistenceCompleteReplicationTaskScope, metrics.PersistenceRequests)
 
@@ -576,15 +612,15 @@ func (p *executionPersistenceClient) GetTimerTask(request *GetTimerTaskRequest)
 	return response, err
 }
 
-func (p *executionPersistenceClient) GetTimerIndexTasks(request *GetTimerIndexTasksRequest) (*GetTimerIndexTasksResponse, error) {
-	p.metricClient.IncCounter(metrics.PersistenceGetTimerIndexTasksScope, metrics.PersistenceRequests)
+func (p *executionPersistenceClient) GetTimerTasks(request *GetTimerTasksRequest) (*GetTimerTasksResponse, error) {
+	p.metricClient.IncCounter(metrics.PersistenceGetTimerTasksScope, metrics.PersistenceRequests)
 
-	sw := p.metricClient.StartTimer(metrics.PersistenceGetTimerIndexTasksScope, metrics.PersistenceLatency)
-	response, err := p.persistence.GetTimerIndexTasks(request)
+	sw := p.metricClient.StartTimer(metrics.PersistenceGetTimerTasksScope, metrics.PersistenceLatency)
+	response, err := p.persistence.GetTimerTasks(request)
 	sw.Stop()
 
 	if err != nil {
-		p.updateErrorMetric(metrics.PersistenceGetTimerIndexTasksScope, err)
+		p.updateErrorMetric(metrics.PersistenceGetTimerTasksScope, err)
 	}
 
 	return response, err
@@ -618,31 +654,6 @@ func (p *executionPersistenceClient) RangeCompleteTimerTask(request *RangeComple
 	return err
 }
 
-func (p *executionPersistenceClient) updateErrorMetric(scope int, err error) {
-	switch err.(type) {
-	case *serviceerror.NotFound:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrEntityNotExistsCounter)
-	case *ShardOwnershipLostError:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrShardOwnershipLostCounter)
-	case *CurrentWorkflowConditionFailedError:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrCurrentWorkflowConditionFailedCounter)
-	case *WorkflowConditionFailedError:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrWorkflowConditionFailedCounter)
-	case *ConditionFailedError:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrConditionFailedCounter)
-	case *TimeoutError:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrTimeoutCounter)
-		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
-	case *serviceerror.ResourceExhausted:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrBusyCounter)
-		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
-	default:
-		p.logger.Error("Operation failed with internal error.",
-			tag.Error(err), tag.MetricScope(scope))
-		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
-	}
-}
-
 func (p *executionPersistenceClient) Close() {
 	p.persistence.Close()
 }
@@ -754,23 +765,6 @@ func (p *taskPersistenceClient) UpdateTaskQueue(request *UpdateTaskQueueRequest)
 	return response, err
 }
 
-func (p *taskPersistenceClient) updateErrorMetric(scope int, err error) {
-	switch err.(type) {
-	case *ConditionFailedError:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrConditionFailedCounter)
-	case *TimeoutError:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrTimeoutCounter)
-		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
-	case *serviceerror.ResourceExhausted:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrBusyCounter)
-		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
-	default:
-		p.logger.Error("Operation failed with internal error.",
-			tag.Error(err), tag.MetricScope(scope))
-		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
-	}
-}
-
 func (p *taskPersistenceClient) Close() {
 	p.persistence.Close()
 }
@@ -881,24 +875,6 @@ func (p *metadataPersistenceClient) Close() {
 	p.persistence.Close()
 }
 
-func (p *metadataPersistenceClient) updateErrorMetric(scope int, err error) {
-	switch err.(type) {
-	case *serviceerror.NamespaceAlreadyExists:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrNamespaceAlreadyExistsCounter)
-	case *serviceerror.NotFound:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrEntityNotExistsCounter)
-	case *serviceerror.InvalidArgument:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrBadRequestCounter)
-	case *serviceerror.ResourceExhausted:
-		p.metricClient.IncCounter(scope, metrics.PersistenceErrBusyCounter)
-		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
-	default:
-		p.logger.Error("Operation failed with internal error.",
-			tag.Error(err), tag.MetricScope(scope))
-		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
-	}
-}
-
 // AppendHistoryNodes add a node to history node table
 func (p *executionPersistenceClient) AppendHistoryNodes(request *AppendHistoryNodesRequest) (*AppendHistoryNodesResponse, error) {
 	p.metricClient.IncCounter(metrics.PersistenceAppendHistoryNodesScope, metrics.PersistenceRequests)
@@ -1172,11 +1148,55 @@ func (c *clusterMetadataPersistenceClient) Close() {
 	c.persistence.Close()
 }
 
-func (c *clusterMetadataPersistenceClient) GetClusterMetadata() (*GetClusterMetadataResponse, error) {
+func (c *clusterMetadataPersistenceClient) ListClusterMetadata(request *ListClusterMetadataRequest) (*ListClusterMetadataResponse, error) {
+	//This is a wrapper of GetClusterMetadata API, use the same scope here
+	c.metricClient.IncCounter(metrics.PersistenceListClusterMetadataScope, metrics.PersistenceRequests)
+
+	sw := c.metricClient.StartTimer(metrics.PersistenceListClusterMetadataScope, metrics.PersistenceLatency)
+	result, err := c.persistence.ListClusterMetadata(request)
+	sw.Stop()
+
+	if err != nil {
+		c.metricClient.IncCounter(metrics.PersistenceListClusterMetadataScope, metrics.PersistenceFailures)
+	}
+
+	return result, err
+}
+
+func (c *clusterMetadataPersistenceClient) GetCurrentClusterMetadata() (*GetClusterMetadataResponse, error) {
+	//This is a wrapper of GetClusterMetadata API, use the same scope here
 	c.metricClient.IncCounter(metrics.PersistenceGetClusterMetadataScope, metrics.PersistenceRequests)
 
 	sw := c.metricClient.StartTimer(metrics.PersistenceGetClusterMetadataScope, metrics.PersistenceLatency)
-	result, err := c.persistence.GetClusterMetadata()
+	result, err := c.persistence.GetCurrentClusterMetadata()
+	sw.Stop()
+
+	if err != nil {
+		c.metricClient.IncCounter(metrics.PersistenceGetClusterMetadataScope, metrics.PersistenceFailures)
+	}
+
+	return result, err
+}
+
+func (c *clusterMetadataPersistenceClient) GetClusterMetadataV1() (*GetClusterMetadataResponse, error) {
+	c.metricClient.IncCounter(metrics.PersistenceGetClusterMetadataScope, metrics.PersistenceRequests)
+
+	sw := c.metricClient.StartTimer(metrics.PersistenceGetClusterMetadataScope, metrics.PersistenceLatency)
+	result, err := c.persistence.GetClusterMetadataV1()
+	sw.Stop()
+
+	if err != nil {
+		c.metricClient.IncCounter(metrics.PersistenceGetClusterMetadataScope, metrics.PersistenceFailures)
+	}
+
+	return result, err
+}
+
+func (c *clusterMetadataPersistenceClient) GetClusterMetadata(request *GetClusterMetadataRequest) (*GetClusterMetadataResponse, error) {
+	c.metricClient.IncCounter(metrics.PersistenceGetClusterMetadataScope, metrics.PersistenceRequests)
+
+	sw := c.metricClient.StartTimer(metrics.PersistenceGetClusterMetadataScope, metrics.PersistenceLatency)
+	result, err := c.persistence.GetClusterMetadata(request)
 	sw.Stop()
 
 	if err != nil {
@@ -1200,6 +1220,20 @@ func (c *clusterMetadataPersistenceClient) SaveClusterMetadata(request *SaveClus
 	return applied, err
 }
 
+func (c *clusterMetadataPersistenceClient) DeleteClusterMetadata(request *DeleteClusterMetadataRequest) error {
+	c.metricClient.IncCounter(metrics.PersistenceDeleteClusterMetadataScope, metrics.PersistenceRequests)
+
+	sw := c.metricClient.StartTimer(metrics.PersistenceDeleteClusterMetadataScope, metrics.PersistenceLatency)
+	err := c.persistence.DeleteClusterMetadata(request)
+	sw.Stop()
+
+	if err != nil {
+		c.metricClient.IncCounter(metrics.PersistenceDeleteClusterMetadataScope, metrics.PersistenceFailures)
+	}
+
+	return err
+}
+
 func (c *clusterMetadataPersistenceClient) GetName() string {
 	return c.persistence.GetName()
 }
@@ -1259,3 +1293,36 @@ func (c *metadataPersistenceClient) InitializeSystemNamespaces(currentClusterNam
 
 	return err
 }
+
+func (p *metricEmitter) updateErrorMetric(scope int, err error) {
+
+	switch err.(type) {
+	case *ShardAlreadyExistError:
+		p.metricClient.IncCounter(scope, metrics.PersistenceErrShardExistsCounter)
+	case *ShardOwnershipLostError:
+		p.metricClient.IncCounter(scope, metrics.PersistenceErrShardOwnershipLostCounter)
+	case *CurrentWorkflowConditionFailedError:
+		p.metricClient.IncCounter(scope, metrics.PersistenceErrCurrentWorkflowConditionFailedCounter)
+	case *WorkflowConditionFailedError:
+		p.metricClient.IncCounter(scope, metrics.PersistenceErrWorkflowConditionFailedCounter)
+	case *ConditionFailedError:
+		p.metricClient.IncCounter(scope, metrics.PersistenceErrConditionFailedCounter)
+	case *TimeoutError:
+		p.metricClient.IncCounter(scope, metrics.PersistenceErrTimeoutCounter)
+		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
+
+	case *serviceerror.InvalidArgument:
+		p.metricClient.IncCounter(scope, metrics.PersistenceErrBadRequestCounter)
+	case *serviceerror.NamespaceAlreadyExists:
+		p.metricClient.IncCounter(scope, metrics.PersistenceErrNamespaceAlreadyExistsCounter)
+	case *serviceerror.NotFound:
+		p.metricClient.IncCounter(scope, metrics.PersistenceErrEntityNotExistsCounter)
+	case *serviceerror.ResourceExhausted:
+		p.metricClient.IncCounter(scope, metrics.PersistenceErrBusyCounter)
+		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
+
+	default:
+		p.logger.Error("Operation failed with internal error.", tag.Error(err), tag.MetricScope(scope))
+		p.metricClient.IncCounter(scope, metrics.PersistenceFailures)
+	}
+}
