@@ -60,6 +60,34 @@ const (
 	testPostgreSQLVisibilitySchema = "../../../schema/postgresql/v96/visibility/schema.sql"
 )
 
+func TestPostgreSQLExecutionMutableStateStoreSuite(t *testing.T) {
+	cfg := NewPostgreSQLConfig()
+	SetupPostgreSQLDatabase(cfg)
+	SetupPostgreSQLSchema(cfg)
+	logger := log.NewNoopLogger()
+	factory := sql.NewFactory(
+		*cfg,
+		resolver.NewNoopResolver(),
+		testPostgreSQLClusterName,
+		logger,
+	)
+	shardStore, err := factory.NewShardStore()
+	if err != nil {
+		t.Fatalf("unable to create PostgreSQL DB: %v", err)
+	}
+	executionStore, err := factory.NewExecutionStore()
+	if err != nil {
+		t.Fatalf("unable to create PostgreSQL DB: %v", err)
+	}
+	defer func() {
+		factory.Close()
+		TearDownPostgreSQLDatabase(cfg)
+	}()
+
+	s := NewExecutionMutableStateSuite(t, shardStore, executionStore, logger)
+	suite.Run(t, s)
+}
+
 func TestPostgreSQLHistoryStoreSuite(t *testing.T) {
 	cfg := NewPostgreSQLConfig()
 	SetupPostgreSQLDatabase(cfg)
@@ -80,7 +108,7 @@ func TestPostgreSQLHistoryStoreSuite(t *testing.T) {
 		TearDownPostgreSQLDatabase(cfg)
 	}()
 
-	s := newHistoryEventsSuite(t, store, logger)
+	s := NewHistoryEventsSuite(t, store, logger)
 	suite.Run(t, s)
 }
 
