@@ -27,11 +27,10 @@ package resource
 import (
 	"net"
 
-	"go.temporal.io/server/common/persistence/serialization"
-	"go.temporal.io/server/common/persistence/visibility"
-
 	"go.temporal.io/api/workflowservice/v1"
-	sdkclient "go.temporal.io/sdk/client"
+
+	"go.temporal.io/server/common/persistence/serialization"
+	"go.temporal.io/server/common/sdk"
 	"go.temporal.io/server/common/searchattribute"
 
 	"go.temporal.io/server/api/adminservice/v1"
@@ -41,12 +40,12 @@ import (
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/archiver"
 	"go.temporal.io/server/common/archiver/provider"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/membership"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	persistenceClient "go.temporal.io/server/common/persistence/client"
 )
@@ -69,7 +68,7 @@ type (
 
 		// other common resources
 
-		GetNamespaceCache() cache.NamespaceCache
+		GetNamespaceRegistry() namespace.Registry
 		GetTimeSource() clock.TimeSource
 		GetPayloadSerializer() serialization.Serializer
 		GetMetricsClient() metrics.Client
@@ -85,8 +84,7 @@ type (
 
 		// internal services clients
 
-		GetSDKClient() sdkclient.Client
-		GetFrontendRawClient() workflowservice.WorkflowServiceClient
+		GetSDKClientFactory() sdk.ClientFactory
 		GetFrontendClient() workflowservice.WorkflowServiceClient
 		GetMatchingRawClient() matchingservice.MatchingServiceClient
 		GetMatchingClient() matchingservice.MatchingServiceClient
@@ -95,17 +93,18 @@ type (
 		GetRemoteAdminClient(cluster string) adminservice.AdminServiceClient
 		GetRemoteFrontendClient(cluster string) workflowservice.WorkflowServiceClient
 		GetClientBean() client.Bean
+		GetClientFactory() client.Factory
 
 		// persistence clients
 
 		GetMetadataManager() persistence.MetadataManager
 		GetClusterMetadataManager() persistence.ClusterMetadataManager
 		GetTaskManager() persistence.TaskManager
-		GetVisibilityManager() visibility.VisibilityManager
 		GetNamespaceReplicationQueue() persistence.NamespaceReplicationQueue
 		GetShardManager() persistence.ShardManager
 		GetExecutionManager() persistence.ExecutionManager
 		GetPersistenceBean() persistenceClient.Bean
+		GetFaultInjection() *persistenceClient.FaultInjectionDataStoreFactory
 
 		// loggers
 
