@@ -12,15 +12,15 @@ else
     export TEMPORAL_CLI_ADDRESS="${BIND_ON_IP}:7233"
 fi
 
-dockerize -template ./config/config_template.yaml:./config/docker.yaml
+dockerize -template /etc/temporal/config/config_template.yaml:/etc/temporal/config/docker.yaml
 
 # Automatically setup Temporal Server (databases, Elasticsearch, default namespace) if "autosetup" is passed as an argument.
-for arg in "$@" ; do [[ ${arg} == "autosetup" ]] && ./auto-setup.sh && break ; done
+for arg in "$@" ; do [[ ${arg} == "autosetup" ]] && /etc/temporal/auto-setup.sh && break ; done
 
 # Setup Temporal Server in development mode if "develop" is passed as an argument.
-for arg in "$@" ; do [[ ${arg} == "develop" ]] && ./setup-develop.sh && break ; done
+for arg in "$@" ; do [[ ${arg} == "develop" ]] && /etc/temporal/setup-develop.sh && break ; done
 
 # Run bash instead of Temporal Server if "bash" is passed as an argument (convenient to debug docker image).
 for arg in "$@" ; do [[ ${arg} == "bash" ]] && bash && exit 0 ; done
 
-./start-temporal.sh
+exec /etc/temporal/start-temporal.sh
