@@ -25,25 +25,26 @@
 package host
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"net"
 	"sync"
 	"time"
 
-	"github.com/uber-go/tally"
+	"github.com/uber-go/tally/v4"
 	"github.com/uber/tchannel-go"
 	"go.temporal.io/api/workflowservice/v1"
-	sdkclient "go.temporal.io/sdk/client"
+	"go.uber.org/fx"
 	"google.golang.org/grpc"
 
 	"go.temporal.io/server/api/adminservice/v1"
 	"go.temporal.io/server/api/historyservice/v1"
+	"go.temporal.io/server/client"
 	"go.temporal.io/server/common"
 	carchiver "go.temporal.io/server/common/archiver"
 	"go.temporal.io/server/common/archiver/provider"
 	"go.temporal.io/server/common/authorization"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/dynamicconfig"
@@ -55,11 +56,11 @@ import (
 	"go.temporal.io/server/common/persistence"
 	persistenceClient "go.temporal.io/server/common/persistence/client"
 	"go.temporal.io/server/common/persistence/visibility"
-	visibilityclient "go.temporal.io/server/common/persistence/visibility/client"
-	esclient "go.temporal.io/server/common/persistence/visibility/elasticsearch/client"
+	esclient "go.temporal.io/server/common/persistence/visibility/store/elasticsearch/client"
 	"go.temporal.io/server/common/resolver"
 	"go.temporal.io/server/common/resource"
 	"go.temporal.io/server/common/rpc"
+	"go.temporal.io/server/common/sdk"
 	"go.temporal.io/server/common/searchattribute"
 	"go.temporal.io/server/service/frontend"
 	"go.temporal.io/server/service/history"
@@ -77,26 +78,35 @@ type Temporal interface {
 	GetFrontendClient() workflowservice.WorkflowServiceClient
 	GetHistoryClient() historyservice.HistoryServiceClient
 	GetExecutionManager() persistence.ExecutionManager
+	RefreshNamespaceCache()
 }
 
 type (
 	temporalImpl struct {
-		frontendService common.Daemon
-		matchingService common.Daemon
-		historyServices []common.Daemon
-		workerService   common.Daemon
+		frontendService *frontend.Service
+		matchingService *matching.Service
+		historyServices []*history.Service
+		workerService   resource.Resource
+
+		frontendApp *fx.App
+		matchingApp *fx.App
+		historyApps []*fx.App
+
+		matchingNamespaceRegistry  namespace.Registry
+		frontendNamespaceRegistry  namespace.Registry
+		historyNamespaceRegistries []namespace.Registry
+		workerNamespaceRegistry    namespace.Registry
 
 		adminClient                      adminservice.AdminServiceClient
 		frontendClient                   workflowservice.WorkflowServiceClient
 		historyClient                    historyservice.HistoryServiceClient
 		logger                           log.Logger
-		clusterMetadataConfig            *config.ClusterMetadata
+		clusterMetadataConfig            *cluster.Config
 		persistenceConfig                config.Persistence
 		metadataMgr                      persistence.MetadataManager
 		clusterMetadataMgr               persistence.ClusterMetadataManager
 		shardMgr                         persistence.ShardManager
 		taskMgr                          persistence.TaskManager
-		visibilityMgr                    visibility.VisibilityManager
 		executionManager                 persistence.ExecutionManager
 		namespaceReplicationQueue        persistence.NamespaceReplicationQueue
 		shutdownCh                       chan struct{}
@@ -107,7 +117,7 @@ type (
 		archiverMetadata                 carchiver.ArchivalMetadata
 		archiverProvider                 provider.ArchiverProvider
 		historyConfig                    *HistoryConfig
-		esConfig                         *config.Elasticsearch
+		esConfig                         *esclient.Config
 		esClient                         esclient.Client
 		workerConfig                     *WorkerConfig
 		mockAdminClient                  map[string]adminservice.AdminServiceClient
@@ -124,14 +134,13 @@ type (
 
 	// TemporalParams contains everything needed to bootstrap Temporal
 	TemporalParams struct {
-		ClusterMetadataConfig            *config.ClusterMetadata
+		ClusterMetadataConfig            *cluster.Config
 		PersistenceConfig                config.Persistence
 		MetadataMgr                      persistence.MetadataManager
 		ClusterMetadataManager           persistence.ClusterMetadataManager
 		ShardMgr                         persistence.ShardManager
 		ExecutionManager                 persistence.ExecutionManager
 		TaskMgr                          persistence.TaskManager
-		VisibilityMgr                    visibility.VisibilityManager
 		NamespaceReplicationQueue        persistence.NamespaceReplicationQueue
 		Logger                           log.Logger
 		ClusterNo                        int
@@ -139,7 +148,7 @@ type (
 		ArchiverProvider                 provider.ArchiverProvider
 		EnableReadHistoryFromArchival    bool
 		HistoryConfig                    *HistoryConfig
-		ESConfig                         *config.Elasticsearch
+		ESConfig                         *esclient.Config
 		ESClient                         esclient.Client
 		WorkerConfig                     *WorkerConfig
 		MockAdminClient                  map[string]adminservice.AdminServiceClient
@@ -153,14 +162,13 @@ type (
 )
 
 // NewTemporal returns an instance that hosts full temporal in one process
-func NewTemporal(params *TemporalParams) Temporal {
+func NewTemporal(params *TemporalParams) *temporalImpl {
 	return &temporalImpl{
 		logger:                           params.Logger,
 		clusterMetadataConfig:            params.ClusterMetadataConfig,
 		persistenceConfig:                params.PersistenceConfig,
 		metadataMgr:                      params.MetadataMgr,
 		clusterMetadataMgr:               params.ClusterMetadataManager,
-		visibilityMgr:                    params.VisibilityMgr,
 		shardMgr:                         params.ShardMgr,
 		taskMgr:                          params.TaskMgr,
 		executionManager:                 params.ExecutionManager,
@@ -223,11 +231,11 @@ func (c *temporalImpl) Stop() {
 	} else {
 		c.shutdownWG.Add(3)
 	}
-	c.frontendService.Stop()
-	for _, historyService := range c.historyServices {
-		historyService.Stop()
+	c.frontendApp.Stop(context.Background())
+	for _, historyApp := range c.historyApps {
+		historyApp.Stop(context.Background())
 	}
-	c.matchingService.Stop()
+	c.matchingApp.Stop(context.Background())
 	if c.workerConfig.EnableReplicator {
 		c.replicator.Stop()
 	}
@@ -369,7 +377,6 @@ func (c *temporalImpl) startFrontend(hosts map[string][]string, startWG *sync.Wa
 	params := &resource.BootstrapParams{}
 	params.DCRedirectionPolicy = config.DCRedirectionPolicy{}
 	params.Name = common.FrontendServiceName
-	params.Logger = c.logger
 	params.ThrottledLogger = c.logger
 	params.RPCFactory = newRPCFactoryImpl(common.FrontendServiceName, c.FrontendGRPCAddress(), c.FrontendRingpopAddress(),
 		c.logger)
@@ -378,36 +385,63 @@ func (c *temporalImpl) startFrontend(hosts map[string][]string, startWG *sync.Wa
 		return newMembershipFactory(params.Name, hosts), nil
 	}
 	params.ClusterMetadataConfig = c.clusterMetadataConfig
-	params.MetricsClient = metrics.NewClient(params.MetricsScope, metrics.GetMetricsServiceIdx(params.Name, c.logger))
-	params.DynamicConfigClient = newIntegrationConfigClient(dynamicconfig.NewNoopClient())
+	params.MetricsClient = metrics.NewClient(&metrics.ClientConfig{}, params.MetricsScope, metrics.GetMetricsServiceIdx(params.Name, c.logger))
 	params.ArchivalMetadata = c.archiverMetadata
 	params.ArchiverProvider = c.archiverProvider
-	params.ESConfig = c.esConfig
-	params.ESClient = c.esClient
-	params.Authorizer = authorization.NewNoopAuthorizer()
 
 	var err error
 	params.PersistenceConfig, err = copyPersistenceConfig(c.persistenceConfig)
 	if err != nil {
 		c.logger.Fatal("Failed to copy persistence config for frontend", tag.Error(err))
 	}
-	params.PersistenceServiceResolver = resolver.NewNoopResolver()
 
 	if c.esConfig != nil {
 		esDataStoreName := "es-visibility"
 		params.PersistenceConfig.AdvancedVisibilityStore = esDataStoreName
 		params.PersistenceConfig.DataStores[esDataStoreName] = config.DataStore{
-			ElasticSearch: c.esConfig,
+			Elasticsearch: c.esConfig,
 		}
 	}
 
-	frontendService, err := frontend.NewService(params)
+	params.SdkClientFactory = sdk.NewClientFactory(
+		c.FrontendGRPCAddress(),
+		nil,
+		params.MetricsScope,
+	)
+
+	stoppedCh := make(chan struct{})
+	var frontendService *frontend.Service
+	var clientBean client.Bean
+	var namespaceRegistry namespace.Registry
+	feApp := fx.New(
+		fx.Supply(
+			params,
+			stoppedCh,
+			params.ClusterMetadataConfig,
+			params.PersistenceConfig,
+		),
+		fx.Provide(func() []grpc.UnaryServerInterceptor { return nil }),
+		fx.Provide(func() authorization.Authorizer { return nil }),
+		fx.Provide(func() authorization.ClaimMapper { return nil }),
+		fx.Provide(func() authorization.JWTAudienceMapper { return nil }),
+		fx.Provide(func() client.FactoryProvider { return client.NewFactoryProvider() }),
+		fx.Provide(func() searchattribute.Mapper { return nil }),
+		fx.Provide(func() resolver.ServiceResolver { return resolver.NewNoopResolver() }),
+		fx.Provide(func() persistenceClient.AbstractDataStoreFactory { return nil }),
+		fx.Provide(func() dynamicconfig.Client { return newIntegrationConfigClient(dynamicconfig.NewNoopClient()) }),
+		fx.Provide(func() log.Logger { return c.logger }),
+		fx.Provide(func() *esclient.Config { return c.esConfig }),
+		fx.Provide(func() esclient.Client { return c.esClient }),
+		frontend.Module,
+		fx.Populate(&frontendService, &clientBean, &namespaceRegistry),
+		fx.NopLogger,
+	)
+	err = feApp.Err()
 	if err != nil {
-		params.Logger.Fatal("unable to start frontend service", tag.Error(err))
+		c.logger.Fatal("unable to construct frontend service", tag.Error(err))
 	}
 
 	if c.mockAdminClient != nil {
-		clientBean := frontendService.GetClientBean()
 		if clientBean != nil {
 			for serviceName, client := range c.mockAdminClient {
 				clientBean.SetRemoteAdminClient(serviceName, client)
@@ -415,11 +449,14 @@ func (c *temporalImpl) startFrontend(hosts map[string][]string, startWG *sync.Wa
 		}
 	}
 
+	c.frontendApp = feApp
 	c.frontendService = frontendService
+	c.frontendNamespaceRegistry = namespaceRegistry
 	connection := params.RPCFactory.CreateFrontendGRPCConnection(c.FrontendGRPCAddress())
 	c.frontendClient = NewFrontendClient(connection)
 	c.adminClient = NewAdminClient(connection)
-	go frontendService.Start()
+
+	feApp.Start(context.Background())
 
 	startWG.Done()
 	<-c.shutdownCh
@@ -434,7 +471,6 @@ func (c *temporalImpl) startHistory(
 	for i, grpcPort := range c.HistoryServiceAddress(3) {
 		params := &resource.BootstrapParams{}
 		params.Name = common.HistoryServiceName
-		params.Logger = c.logger
 		params.ThrottledLogger = c.logger
 		params.RPCFactory = newRPCFactoryImpl(common.HistoryServiceName, grpcPort, membershipPorts[i], c.logger)
 		params.MetricsScope = tally.NewTestScope(common.HistoryServiceName, make(map[string]string))
@@ -442,50 +478,62 @@ func (c *temporalImpl) startHistory(
 			return newMembershipFactory(params.Name, hosts), nil
 		}
 		params.ClusterMetadataConfig = c.clusterMetadataConfig
-		params.MetricsClient = metrics.NewClient(params.MetricsScope, metrics.GetMetricsServiceIdx(params.Name, c.logger))
+		params.MetricsClient = metrics.NewClient(&metrics.ClientConfig{}, params.MetricsScope, metrics.GetMetricsServiceIdx(params.Name, c.logger))
 		integrationClient := newIntegrationConfigClient(dynamicconfig.NewNoopClient())
 		c.overrideHistoryDynamicConfig(integrationClient)
-		params.DynamicConfigClient = integrationClient
 
-		var err error
-		params.SdkClient, err = sdkclient.NewClient(sdkclient.Options{
-			HostPort:     c.FrontendGRPCAddress(),
-			Namespace:    common.SystemLocalNamespace,
-			MetricsScope: params.MetricsScope,
-			ConnectionOptions: sdkclient.ConnectionOptions{
-				DisableHealthCheck: true,
-			},
-		})
-		if err != nil {
-			c.logger.Fatal("Failed to create client for history", tag.Error(err))
-		}
+		params.SdkClientFactory = sdk.NewClientFactory(
+			c.FrontendGRPCAddress(),
+			nil,
+			params.MetricsScope,
+		)
 
 		params.ArchivalMetadata = c.archiverMetadata
 		params.ArchiverProvider = c.archiverProvider
-		params.ESConfig = c.esConfig
-		params.ESClient = c.esClient
 
+		var err error
 		params.PersistenceConfig, err = copyPersistenceConfig(c.persistenceConfig)
 		if err != nil {
 			c.logger.Fatal("Failed to copy persistence config for history", tag.Error(err))
 		}
-		params.PersistenceServiceResolver = resolver.NewNoopResolver()
 
 		if c.esConfig != nil {
 			esDataStoreName := "es-visibility"
 			params.PersistenceConfig.AdvancedVisibilityStore = esDataStoreName
 			params.PersistenceConfig.DataStores[esDataStoreName] = config.DataStore{
-				ElasticSearch: c.esConfig,
+				Elasticsearch: c.esConfig,
 			}
 		}
 
-		historyService, err := history.NewService(params)
+		stoppedCh := make(chan struct{})
+		var historyService *history.Service
+		var clientBean client.Bean
+		var namespaceRegistry namespace.Registry
+		app := fx.New(
+			fx.Supply(
+				params,
+				stoppedCh,
+				integrationClient,
+				params.ClusterMetadataConfig,
+				params.PersistenceConfig,
+			),
+			fx.Provide(func() client.FactoryProvider { return client.NewFactoryProvider() }),
+			fx.Provide(func() searchattribute.Mapper { return nil }),
+			fx.Provide(func() resolver.ServiceResolver { return resolver.NewNoopResolver() }),
+			fx.Provide(func() persistenceClient.AbstractDataStoreFactory { return nil }),
+			fx.Provide(func() dynamicconfig.Client { return integrationClient }),
+			fx.Provide(func() log.Logger { return c.logger }),
+			fx.Provide(func() *esclient.Config { return c.esConfig }),
+			fx.Provide(func() esclient.Client { return c.esClient }),
+			history.Module,
+			fx.Populate(&historyService, &clientBean, &namespaceRegistry),
+			fx.NopLogger)
+		err = app.Err()
 		if err != nil {
-			params.Logger.Fatal("unable to start history service", tag.Error(err))
+			c.logger.Fatal("unable to construct history service", tag.Error(err))
 		}
 
 		if c.mockAdminClient != nil {
-			clientBean := historyService.GetClientBean()
 			if clientBean != nil {
 				for serviceName, client := range c.mockAdminClient {
 					clientBean.SetRemoteAdminClient(serviceName, client)
@@ -502,10 +550,12 @@ func (c *temporalImpl) startHistory(
 			c.logger.Fatal("Failed to create connection for history", tag.Error(err))
 		}
 
+		c.historyApps = append(c.historyApps, app)
 		c.historyClient = NewHistoryClient(historyConnection)
 		c.historyServices = append(c.historyServices, historyService)
+		c.historyNamespaceRegistries = append(c.historyNamespaceRegistries, namespaceRegistry)
 
-		go historyService.Start()
+		app.Start(context.Background())
 	}
 
 	startWG.Done()
@@ -514,10 +564,8 @@ func (c *temporalImpl) startHistory(
 }
 
 func (c *temporalImpl) startMatching(hosts map[string][]string, startWG *sync.WaitGroup) {
-
 	params := &resource.BootstrapParams{}
 	params.Name = common.MatchingServiceName
-	params.Logger = c.logger
 	params.ThrottledLogger = c.logger
 	params.RPCFactory = newRPCFactoryImpl(common.MatchingServiceName, c.MatchingGRPCServiceAddress(), c.MatchingServiceRingpopAddress(), c.logger)
 	params.MetricsScope = tally.NewTestScope(common.MatchingServiceName, make(map[string]string))
@@ -525,8 +573,7 @@ func (c *temporalImpl) startMatching(hosts map[string][]string, startWG *sync.Wa
 		return newMembershipFactory(params.Name, hosts), nil
 	}
 	params.ClusterMetadataConfig = c.clusterMetadataConfig
-	params.MetricsClient = metrics.NewClient(params.MetricsScope, metrics.GetMetricsServiceIdx(params.Name, c.logger))
-	params.DynamicConfigClient = newIntegrationConfigClient(dynamicconfig.NewNoopClient())
+	params.MetricsClient = metrics.NewClient(&metrics.ClientConfig{}, params.MetricsScope, metrics.GetMetricsServiceIdx(params.Name, c.logger))
 	params.ArchivalMetadata = c.archiverMetadata
 	params.ArchiverProvider = c.archiverProvider
 
@@ -535,22 +582,42 @@ func (c *temporalImpl) startMatching(hosts map[string][]string, startWG *sync.Wa
 	if err != nil {
 		c.logger.Fatal("Failed to copy persistence config for matching", tag.Error(err))
 	}
-	params.PersistenceServiceResolver = resolver.NewNoopResolver()
 
-	matchingService, err := matching.NewService(params)
+	stoppedCh := make(chan struct{})
+	var matchingService *matching.Service
+	var clientBean client.Bean
+	var namespaceRegistry namespace.Registry
+	app := fx.New(
+		fx.Supply(
+			stoppedCh,
+			params,
+			params.ClusterMetadataConfig,
+		),
+		fx.Provide(func() client.FactoryProvider { return client.NewFactoryProvider() }),
+		fx.Provide(func() searchattribute.Mapper { return nil }),
+		fx.Provide(func() resolver.ServiceResolver { return resolver.NewNoopResolver() }),
+		fx.Provide(func() persistenceClient.AbstractDataStoreFactory { return nil }),
+		fx.Provide(func() dynamicconfig.Client { return newIntegrationConfigClient(dynamicconfig.NewNoopClient()) }),
+		fx.Provide(func() log.Logger { return c.logger }),
+		matching.Module,
+		fx.Populate(&matchingService, &clientBean, &namespaceRegistry),
+		fx.NopLogger,
+	)
+	err = app.Err()
 	if err != nil {
-		params.Logger.Fatal("unable to start matching service", tag.Error(err))
+		c.logger.Fatal("unable to start matching service", tag.Error(err))
 	}
 	if c.mockAdminClient != nil {
-		clientBean := matchingService.GetClientBean()
 		if clientBean != nil {
 			for serviceName, client := range c.mockAdminClient {
 				clientBean.SetRemoteAdminClient(serviceName, client)
 			}
 		}
 	}
+	c.matchingApp = app
 	c.matchingService = matchingService
-	go c.matchingService.Start()
+	c.matchingNamespaceRegistry = namespaceRegistry
+	app.Start(context.Background())
 
 	startWG.Done()
 	<-c.shutdownCh
@@ -558,9 +625,9 @@ func (c *temporalImpl) startMatching(hosts map[string][]string, startWG *sync.Wa
 }
 
 func (c *temporalImpl) startWorker(hosts map[string][]string, startWG *sync.WaitGroup) {
+
 	params := &resource.BootstrapParams{}
 	params.Name = common.WorkerServiceName
-	params.Logger = c.logger
 	params.ThrottledLogger = c.logger
 	params.RPCFactory = newRPCFactoryImpl(common.WorkerServiceName, c.WorkerGRPCServiceAddress(), c.WorkerServiceRingpopAddress(), c.logger)
 	params.MetricsScope = tally.NewTestScope(common.WorkerServiceName, make(map[string]string))
@@ -568,8 +635,7 @@ func (c *temporalImpl) startWorker(hosts map[string][]string, startWG *sync.Wait
 		return newMembershipFactory(params.Name, hosts), nil
 	}
 	params.ClusterMetadataConfig = c.clusterMetadataConfig
-	params.MetricsClient = metrics.NewClient(params.MetricsScope, metrics.GetMetricsServiceIdx(params.Name, c.logger))
-	params.DynamicConfigClient = newIntegrationConfigClient(dynamicconfig.NewNoopClient())
+	params.MetricsClient = metrics.NewClient(&metrics.ClientConfig{}, params.MetricsScope, metrics.GetMetricsServiceIdx(params.Name, c.logger))
 	params.ArchivalMetadata = c.archiverMetadata
 	params.ArchiverProvider = c.archiverProvider
 
@@ -578,66 +644,48 @@ func (c *temporalImpl) startWorker(hosts map[string][]string, startWG *sync.Wait
 	if err != nil {
 		c.logger.Fatal("Failed to copy persistence config for worker", tag.Error(err))
 	}
-	params.PersistenceServiceResolver = resolver.NewNoopResolver()
 
-	params.SdkClient, err = sdkclient.NewClient(sdkclient.Options{
-		HostPort:     c.FrontendGRPCAddress(),
-		Namespace:    common.SystemLocalNamespace,
-		MetricsScope: params.MetricsScope,
-		ConnectionOptions: sdkclient.ConnectionOptions{
-			DisableHealthCheck: true,
-		},
-	})
-	if err != nil {
-		c.logger.Fatal("Failed to create client for worker", tag.Error(err))
-	}
+	params.SdkClientFactory = sdk.NewClientFactory(
+		c.FrontendGRPCAddress(),
+		nil,
+		params.MetricsScope,
+	)
 
+	dcClient := newIntegrationConfigClient(dynamicconfig.NewNoopClient())
 	service, err := resource.New(
+		c.logger,
 		params,
 		common.WorkerServiceName,
+		dcClient,
 		dynamicconfig.GetIntPropertyFn(5000),
 		dynamicconfig.GetIntPropertyFn(5000),
 		dynamicconfig.GetIntPropertyFn(10000),
-		func(
-			persistenceBean persistenceClient.Bean,
-			searchAttributesProvider searchattribute.Provider,
-			logger log.Logger,
-		) (visibility.VisibilityManager, error) {
-			visibilityFromDB, err := visibilityclient.NewVisibilityManager(
-				params.PersistenceConfig,
-				dynamicconfig.GetIntPropertyFn(5000),
-				params.MetricsClient,
-				params.PersistenceServiceResolver,
-				params.Logger,
-			)
-			if err != nil {
-				return nil, err
-			}
-
-			return visibilityFromDB, nil
-		},
+		nil,
+		resolver.NewNoopResolver(),
+		nil,
 	)
+
 	if err != nil {
-		params.Logger.Fatal("unable to create worker service", tag.Error(err))
+		c.logger.Fatal("unable to create worker service", tag.Error(err))
 	}
 	c.workerService = service
 	service.Start()
 
-	clusterMetadata := cluster.NewTestClusterMetadata(c.clusterMetadataConfig)
-	var replicatorNamespaceCache cache.NamespaceCache
+	clusterMetadata := cluster.NewMetadataFromConfig(c.clusterMetadataConfig, c.clusterMetadataMgr, c.logger)
+	var replicatorNamespaceCache namespace.Registry
 	if c.workerConfig.EnableReplicator {
 		metadataManager := persistence.NewMetadataPersistenceMetricsClient(c.metadataMgr, service.GetMetricsClient(), c.logger)
-		replicatorNamespaceCache = cache.NewNamespaceCache(metadataManager, clusterMetadata, service.GetMetricsClient(), service.GetLogger())
+		replicatorNamespaceCache = namespace.NewRegistry(metadataManager, clusterMetadata.IsGlobalNamespaceEnabled(), service.GetMetricsClient(), service.GetLogger())
 		replicatorNamespaceCache.Start()
 		c.startWorkerReplicator(service, clusterMetadata)
 	}
 
-	var clientWorkerNamespaceCache cache.NamespaceCache
+	var clientWorkerNamespaceCache namespace.Registry
 	if c.workerConfig.EnableArchiver {
 		metadataProxyManager := persistence.NewMetadataPersistenceMetricsClient(c.metadataMgr, service.GetMetricsClient(), c.logger)
-		clientWorkerNamespaceCache = cache.NewNamespaceCache(metadataProxyManager, clusterMetadata, service.GetMetricsClient(), service.GetLogger())
+		clientWorkerNamespaceCache = namespace.NewRegistry(metadataProxyManager, clusterMetadata.IsGlobalNamespaceEnabled(), service.GetMetricsClient(), service.GetLogger())
 		clientWorkerNamespaceCache.Start()
-		c.startWorkerClientWorker(params, service, clientWorkerNamespaceCache)
+		c.startWorkerClientWorker(params, service, clientWorkerNamespaceCache, dcClient)
 	}
 
 	startWG.Done()
@@ -669,16 +717,25 @@ func (c *temporalImpl) startWorkerReplicator(service resource.Resource, clusterM
 	c.replicator.Start()
 }
 
-func (c *temporalImpl) startWorkerClientWorker(params *resource.BootstrapParams, service resource.Resource, namespaceCache cache.NamespaceCache) {
-	workerConfig := worker.NewConfig(params)
+func (c *temporalImpl) startWorkerClientWorker(params *resource.BootstrapParams, service resource.Resource, namespaceRegistry namespace.Registry, dcClient dynamicconfig.Client) {
+	workerConfig := worker.NewConfig(
+		c.logger,
+		dcClient,
+		params,
+	)
 	workerConfig.ArchiverConfig.ArchiverConcurrency = dynamicconfig.GetIntPropertyFn(10)
 
+	systemSdkClient, err := params.SdkClientFactory.NewSystemClient(c.logger)
+	if err != nil {
+		c.logger.Fatal("Unable to create SDK client", tag.Error(err))
+	}
+
 	bc := &archiver.BootstrapContainer{
-		SdkClient:        params.SdkClient,
+		SdkSystemClient:  systemSdkClient,
 		MetricsClient:    service.GetMetricsClient(),
 		Logger:           c.logger,
 		HistoryV2Manager: c.executionManager,
-		NamespaceCache:   namespaceCache,
+		NamespaceCache:   namespaceRegistry,
 		Config:           workerConfig.ArchiverConfig,
 		ArchiverProvider: c.archiverProvider,
 	}
@@ -705,7 +762,7 @@ func (c *temporalImpl) overrideHistoryDynamicConfig(client *dynamicClient) {
 	client.OverrideValue(dynamicconfig.ReplicationTaskProcessorStartWait, time.Nanosecond)
 
 	if c.workerConfig.EnableIndexer {
-		client.OverrideValue(dynamicconfig.AdvancedVisibilityWritingMode, common.AdvancedVisibilityWritingModeDual)
+		client.OverrideValue(dynamicconfig.AdvancedVisibilityWritingMode, visibility.AdvancedVisibilityWritingModeDual)
 	}
 	if c.historyConfig.HistoryCountLimitWarn != 0 {
 		client.OverrideValue(dynamicconfig.HistoryCountLimitWarn, c.historyConfig.HistoryCountLimitWarn)
@@ -715,6 +772,17 @@ func (c *temporalImpl) overrideHistoryDynamicConfig(client *dynamicClient) {
 	}
 }
 
+func (c *temporalImpl) RefreshNamespaceCache() {
+	c.frontendNamespaceRegistry.Refresh()
+	c.matchingNamespaceRegistry.Refresh()
+	for _, r := range c.historyNamespaceRegistries {
+		r.Refresh()
+	}
+	if c.workerNamespaceRegistry != nil {
+		c.workerNamespaceRegistry.Refresh()
+	}
+}
+
 // copyPersistenceConfig makes a deepcopy of persistence config.
 // This is just a temp fix for the race condition of persistence config.
 // The race condition happens because all the services are using the same datastore map in the config.
