@@ -26,53 +26,49 @@ package frontend
 
 import (
 	"math"
+	"math/rand"
+	"net"
 	"os"
 	"sync/atomic"
 	"time"
 
+	"github.com/uber-go/tally/v4"
 	"go.temporal.io/api/workflowservice/v1"
+	"google.golang.org/grpc"
+	healthpb "google.golang.org/grpc/health/grpc_health_v1"
+	"google.golang.org/grpc/reflection"
+
 	"go.temporal.io/server/api/adminservice/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/authorization"
-	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/membership"
 	"go.temporal.io/server/common/metrics"
 	"go.temporal.io/server/common/namespace"
-	"go.temporal.io/server/common/persistence"
-	persistenceClient "go.temporal.io/server/common/persistence/client"
-	"go.temporal.io/server/common/persistence/visibility"
-	visibilityclient "go.temporal.io/server/common/persistence/visibility/client"
-	"go.temporal.io/server/common/persistence/visibility/elasticsearch"
-	"go.temporal.io/server/common/quotas"
-	"go.temporal.io/server/common/resource"
-	"go.temporal.io/server/common/rpc"
-	"go.temporal.io/server/common/rpc/interceptor"
-	"go.temporal.io/server/common/searchattribute"
-	"go.temporal.io/server/service/frontend/configs"
-	"google.golang.org/grpc"
-	healthpb "google.golang.org/grpc/health/grpc_health_v1"
-	"google.golang.org/grpc/keepalive"
-	"google.golang.org/grpc/reflection"
+	"go.temporal.io/server/common/persistence/client"
+	"go.temporal.io/server/common/persistence/visibility/manager"
 )
 
 // Config represents configuration for frontend service
 type Config struct {
-	NumHistoryShards             int32
-	ESIndexName                  string
-	PersistenceMaxQPS            dynamicconfig.IntPropertyFn
-	PersistenceGlobalMaxQPS      dynamicconfig.IntPropertyFn
-	VisibilityMaxPageSize        dynamicconfig.IntPropertyFnWithNamespaceFilter
-	EnableVisibilitySampling     dynamicconfig.BoolPropertyFn
-	VisibilityListMaxQPS         dynamicconfig.IntPropertyFnWithNamespaceFilter
-	EnableReadVisibilityFromES   dynamicconfig.BoolPropertyFnWithNamespaceFilter
-	ESVisibilityListMaxQPS       dynamicconfig.IntPropertyFnWithNamespaceFilter
-	ESIndexMaxResultWindow       dynamicconfig.IntPropertyFn
+	NumHistoryShards        int32
+	ESIndexName             string
+	PersistenceMaxQPS       dynamicconfig.IntPropertyFn
+	PersistenceGlobalMaxQPS dynamicconfig.IntPropertyFn
+
+	StandardVisibilityPersistenceMaxReadQPS  dynamicconfig.IntPropertyFn
+	StandardVisibilityPersistenceMaxWriteQPS dynamicconfig.IntPropertyFn
+	AdvancedVisibilityPersistenceMaxReadQPS  dynamicconfig.IntPropertyFn
+	AdvancedVisibilityPersistenceMaxWriteQPS dynamicconfig.IntPropertyFn
+	VisibilityMaxPageSize                    dynamicconfig.IntPropertyFnWithNamespaceFilter
+	EnableReadVisibilityFromES               dynamicconfig.BoolPropertyFnWithNamespaceFilter
+	ESIndexMaxResultWindow                   dynamicconfig.IntPropertyFn
+
 	HistoryMaxPageSize           dynamicconfig.IntPropertyFnWithNamespaceFilter
 	RPS                          dynamicconfig.IntPropertyFn
 	MaxNamespaceRPSPerInstance   dynamicconfig.IntPropertyFnWithNamespaceFilter
+	MaxNamespaceBurstPerInstance dynamicconfig.IntPropertyFnWithNamespaceFilter
 	MaxNamespaceCountPerInstance dynamicconfig.IntPropertyFnWithNamespaceFilter
 	GlobalNamespaceRPS           dynamicconfig.IntPropertyFnWithNamespaceFilter
 	MaxIDLengthLimit             dynamicconfig.IntPropertyFn
@@ -136,19 +132,23 @@ type Config struct {
 // NewConfig returns new service config with default values
 func NewConfig(dc *dynamicconfig.Collection, numHistoryShards int32, esIndexName string, enableReadFromES bool) *Config {
 	return &Config{
-		NumHistoryShards:                       numHistoryShards,
-		ESIndexName:                            esIndexName,
-		PersistenceMaxQPS:                      dc.GetIntProperty(dynamicconfig.FrontendPersistenceMaxQPS, 2000),
-		PersistenceGlobalMaxQPS:                dc.GetIntProperty(dynamicconfig.FrontendPersistenceGlobalMaxQPS, 0),
-		VisibilityMaxPageSize:                  dc.GetIntPropertyFilteredByNamespace(dynamicconfig.FrontendVisibilityMaxPageSize, 1000),
-		EnableVisibilitySampling:               dc.GetBoolProperty(dynamicconfig.EnableVisibilitySampling, true),
-		VisibilityListMaxQPS:                   dc.GetIntPropertyFilteredByNamespace(dynamicconfig.FrontendVisibilityListMaxQPS, 30),
-		EnableReadVisibilityFromES:             dc.GetBoolPropertyFnWithNamespaceFilter(dynamicconfig.EnableReadVisibilityFromES, enableReadFromES),
-		ESVisibilityListMaxQPS:                 dc.GetIntPropertyFilteredByNamespace(dynamicconfig.FrontendESVisibilityListMaxQPS, 10),
-		ESIndexMaxResultWindow:                 dc.GetIntProperty(dynamicconfig.FrontendESIndexMaxResultWindow, 10000),
+		NumHistoryShards:        numHistoryShards,
+		ESIndexName:             esIndexName,
+		PersistenceMaxQPS:       dc.GetIntProperty(dynamicconfig.FrontendPersistenceMaxQPS, 2000),
+		PersistenceGlobalMaxQPS: dc.GetIntProperty(dynamicconfig.FrontendPersistenceGlobalMaxQPS, 0),
+
+		StandardVisibilityPersistenceMaxReadQPS:  dc.GetIntProperty(dynamicconfig.StandardVisibilityPersistenceMaxReadQPS, 9000),
+		StandardVisibilityPersistenceMaxWriteQPS: dc.GetIntProperty(dynamicconfig.StandardVisibilityPersistenceMaxWriteQPS, 9000),
+		AdvancedVisibilityPersistenceMaxReadQPS:  dc.GetIntProperty(dynamicconfig.AdvancedVisibilityPersistenceMaxReadQPS, 9000),
+		AdvancedVisibilityPersistenceMaxWriteQPS: dc.GetIntProperty(dynamicconfig.AdvancedVisibilityPersistenceMaxWriteQPS, 9000),
+		VisibilityMaxPageSize:                    dc.GetIntPropertyFilteredByNamespace(dynamicconfig.FrontendVisibilityMaxPageSize, 1000),
+		EnableReadVisibilityFromES:               dc.GetBoolPropertyFnWithNamespaceFilter(dynamicconfig.EnableReadVisibilityFromES, enableReadFromES),
+		ESIndexMaxResultWindow:                   dc.GetIntProperty(dynamicconfig.FrontendESIndexMaxResultWindow, 10000),
+
 		HistoryMaxPageSize:                     dc.GetIntPropertyFilteredByNamespace(dynamicconfig.FrontendHistoryMaxPageSize, common.GetHistoryMaxPageSize),
 		RPS:                                    dc.GetIntProperty(dynamicconfig.FrontendRPS, 2400),
 		MaxNamespaceRPSPerInstance:             dc.GetIntPropertyFilteredByNamespace(dynamicconfig.FrontendMaxNamespaceRPSPerInstance, 2400),
+		MaxNamespaceBurstPerInstance:           dc.GetIntPropertyFilteredByNamespace(dynamicconfig.FrontendMaxNamespaceBurstPerInstance, 4800),
 		MaxNamespaceCountPerInstance:           dc.GetIntPropertyFilteredByNamespace(dynamicconfig.FrontendMaxNamespaceCountPerInstance, 1200),
 		GlobalNamespaceRPS:                     dc.GetIntPropertyFilteredByNamespace(dynamicconfig.FrontendGlobalNamespaceRPS, 0),
 		MaxIDLengthLimit:                       dc.GetIntProperty(dynamicconfig.MaxIDLengthLimit, 1000),
@@ -182,175 +182,48 @@ func NewConfig(dc *dynamicconfig.Collection, numHistoryShards int32, esIndexName
 
 // Service represents the frontend service
 type Service struct {
-	resource.Resource
-
 	status int32
 	config *Config
 
-	handler        Handler
-	adminHandler   *AdminHandler
-	versionChecker *VersionChecker
-	server         *grpc.Server
+	handler           Handler
+	adminHandler      *AdminHandler
+	versionChecker    *VersionChecker
+	visibilityManager manager.VisibilityManager
+	server            *grpc.Server
 
-	serverMetricsReporter metrics.Reporter
-	sdkMetricsReporter    metrics.Reporter
+	serverMetricsReporter          metrics.Reporter
+	sdkMetricsReporter             metrics.Reporter
+	logger                         log.Logger
+	grpcListener                   net.Listener
+	metricsScope                   tally.Scope
+	faultInjectionDataStoreFactory *client.FaultInjectionDataStoreFactory
 }
 
-// NewService builds a new frontend service
 func NewService(
-	params *resource.BootstrapParams,
-) (*Service, error) {
-
-	isAdvancedVisExistInConfig := len(params.PersistenceConfig.AdvancedVisibilityStore) != 0
-	serviceConfig := NewConfig(
-		dynamicconfig.NewCollection(params.DynamicConfigClient, params.Logger),
-		params.PersistenceConfig.NumHistoryShards,
-		params.ESConfig.GetVisibilityIndex(),
-		isAdvancedVisExistInConfig)
-
-	params.PersistenceConfig.VisibilityConfig = &config.VisibilityConfig{
-		VisibilityListMaxQPS: serviceConfig.VisibilityListMaxQPS,
-		EnableSampling:       serviceConfig.EnableVisibilitySampling,
-	}
-
-	visibilityManagerInitializer := func(
-		persistenceBean persistenceClient.Bean,
-		searchAttributesProvider searchattribute.Provider,
-		logger log.Logger,
-	) (visibility.VisibilityManager, error) {
-		visibilityFromDB, err := visibilityclient.NewVisibilityManager(
-			params.PersistenceConfig,
-			serviceConfig.PersistenceMaxQPS,
-			params.MetricsClient,
-			params.PersistenceServiceResolver,
-			params.Logger,
-		)
-		if err != nil {
-			return nil, err
-		}
-
-		var visibilityFromES visibility.VisibilityManager
-		if params.ESConfig != nil {
-			visibilityIndexName := params.ESConfig.GetVisibilityIndex()
-			visibilityConfigForES := &config.VisibilityConfig{
-				MaxQPS:               serviceConfig.PersistenceMaxQPS,
-				VisibilityListMaxQPS: serviceConfig.ESVisibilityListMaxQPS,
-			}
-			visibilityFromES = elasticsearch.NewVisibilityManager(visibilityIndexName, params.ESClient, visibilityConfigForES,
-				searchAttributesProvider, params.SearchAttributesMapper, nil, params.MetricsClient, logger)
-		}
-		return visibility.NewVisibilityManagerWrapper(
-			visibilityFromDB,
-			visibilityFromES,
-			serviceConfig.EnableReadVisibilityFromES,
-			dynamicconfig.GetStringPropertyFn(common.AdvancedVisibilityWritingModeOff), // frontend visibility never write
-		), nil
-	}
-
-	serviceResource, err := resource.New(
-		params,
-		common.FrontendServiceName,
-		serviceConfig.PersistenceMaxQPS,
-		serviceConfig.PersistenceGlobalMaxQPS,
-		serviceConfig.ThrottledLogRPS,
-		visibilityManagerInitializer,
-	)
-	if err != nil {
-		return nil, err
-	}
-
-	var namespaceReplicationQueue persistence.NamespaceReplicationQueue
-	clusterMetadata := serviceResource.GetClusterMetadata()
-	if clusterMetadata.IsGlobalNamespaceEnabled() {
-		namespaceReplicationQueue = serviceResource.GetNamespaceReplicationQueue()
-	}
-
-	metricsInterceptor := interceptor.NewTelemetryInterceptor(
-		serviceResource.GetNamespaceCache(),
-		serviceResource.GetMetricsClient(),
-		metrics.FrontendAPIMetricsScopes(),
-		serviceResource.GetLogger(),
-	)
-	rateLimiterInterceptor := interceptor.NewRateLimitInterceptor(
-		configs.NewRequestToRateLimiter(func() float64 { return float64(serviceConfig.RPS()) }),
-		map[string]int{},
-	)
-	namespaceRateLimiterInterceptor := interceptor.NewNamespaceRateLimitInterceptor(
-		serviceResource.GetNamespaceCache(),
-		quotas.NewNamespaceRateLimiter(
-			func(req quotas.Request) quotas.RequestRateLimiter {
-				return configs.NewRequestToRateLimiter(func() float64 {
-					return namespaceRPS(
-						serviceConfig,
-						serviceResource.GetFrontendServiceResolver(),
-						req.Caller,
-					)
-				})
-			},
-		),
-		map[string]int{},
-	)
-	namespaceCountLimiterInterceptor := interceptor.NewNamespaceCountLimitInterceptor(
-		serviceResource.GetNamespaceCache(),
-		serviceConfig.MaxNamespaceCountPerInstance,
-		configs.ExecutionAPICountLimitOverride,
-	)
-
-	namespaceLogger := params.NamespaceLogger
-	namespaceLogInterceptor := interceptor.NewNamespaceLogInterceptor(
-		serviceResource.GetNamespaceCache(),
-		namespaceLogger)
-
-	kep := keepalive.EnforcementPolicy{
-		MinTime:             serviceConfig.KeepAliveMinTime(),
-		PermitWithoutStream: serviceConfig.KeepAlivePermitWithoutStream(),
-	}
-	var kp = keepalive.ServerParameters{
-		MaxConnectionIdle:     serviceConfig.KeepAliveMaxConnectionIdle(),
-		MaxConnectionAge:      serviceConfig.KeepAliveMaxConnectionAge(),
-		MaxConnectionAgeGrace: serviceConfig.KeepAliveMaxConnectionAgeGrace(),
-		Time:                  serviceConfig.KeepAliveTime(),
-		Timeout:               serviceConfig.KeepAliveTimeout(),
-	}
-
-	grpcServerOptions, err := params.RPCFactory.GetFrontendGRPCServerOptions()
-	if err != nil {
-		params.Logger.Fatal("creating gRPC server options failed", tag.Error(err))
-	}
-	grpcServerOptions = append(
-		grpcServerOptions,
-		grpc.KeepaliveParams(kp),
-		grpc.KeepaliveEnforcementPolicy(kep),
-		grpc.ChainUnaryInterceptor(
-			namespaceLogInterceptor.Intercept,
-			rpc.ServiceErrorInterceptor,
-			metrics.NewServerMetricsContextInjectorInterceptor(),
-			metricsInterceptor.Intercept,
-			rateLimiterInterceptor.Intercept,
-			namespaceRateLimiterInterceptor.Intercept,
-			namespaceCountLimiterInterceptor.Intercept,
-			authorization.NewAuthorizationInterceptor(
-				params.ClaimMapper,
-				params.Authorizer,
-				serviceResource.GetMetricsClient(),
-				params.Logger,
-				params.AudienceGetter,
-			),
-		),
-	)
-
-	wfHandler := NewWorkflowHandler(serviceResource, serviceConfig, namespaceReplicationQueue)
-	handler := NewDCRedirectionHandler(wfHandler, params.DCRedirectionPolicy)
-
+	serviceConfig *Config,
+	server *grpc.Server,
+	handler Handler,
+	adminHandler *AdminHandler,
+	versionChecker *VersionChecker,
+	visibilityMgr manager.VisibilityManager,
+	logger log.Logger,
+	grpcListener net.Listener,
+	metricsScope tally.Scope,
+	faultInjectionDataStoreFactory *client.FaultInjectionDataStoreFactory,
+) *Service {
 	return &Service{
-		Resource:       serviceResource,
-		status:         common.DaemonStatusInitialized,
-		config:         serviceConfig,
-		server:         grpc.NewServer(grpcServerOptions...),
-		handler:        handler,
-		adminHandler:   NewAdminHandler(serviceResource, params, serviceConfig),
-		versionChecker: NewVersionChecker(serviceConfig, params.MetricsClient, serviceResource.GetClusterMetadataManager()),
-	}, nil
+		status:                         common.DaemonStatusInitialized,
+		config:                         serviceConfig,
+		server:                         server,
+		handler:                        handler,
+		adminHandler:                   adminHandler,
+		versionChecker:                 versionChecker,
+		visibilityManager:              visibilityMgr,
+		logger:                         logger,
+		grpcListener:                   grpcListener,
+		metricsScope:                   metricsScope,
+		faultInjectionDataStoreFactory: faultInjectionDataStoreFactory,
+	}
 }
 
 // Start starts the service
@@ -359,7 +232,7 @@ func (s *Service) Start() {
 		return
 	}
 
-	logger := s.GetLogger()
+	logger := s.logger
 	logger.Info("frontend starting")
 
 	workflowservice.RegisterWorkflowServiceServer(s.server, s.handler)
@@ -370,11 +243,13 @@ func (s *Service) Start() {
 	reflection.Register(s.server)
 
 	// must start resource first
-	s.Resource.Start()
+	s.metricsScope.Counter(metrics.RestartCount).Inc(1)
+	rand.Seed(time.Now().UnixNano())
+
 	s.adminHandler.Start()
 	s.versionChecker.Start()
 
-	listener := s.GetGRPCListener()
+	listener := s.grpcListener
 	logger.Info("Starting to serve on frontend listener")
 	if err := s.server.Serve(listener); err != nil {
 		logger.Fatal("Failed to serve on frontend listener", tag.Error(err))
@@ -383,7 +258,7 @@ func (s *Service) Start() {
 
 // Stop stops the service
 func (s *Service) Stop() {
-	logger := s.GetLogger()
+	logger := s.logger
 
 	if !atomic.CompareAndSwapInt32(&s.status, common.DaemonStatusStarted, common.DaemonStatusStopped) {
 		return
@@ -407,13 +282,13 @@ func (s *Service) Stop() {
 
 	s.adminHandler.Stop()
 	s.versionChecker.Stop()
+	s.visibilityManager.Close()
 
 	logger.Info("ShutdownHandler: Draining traffic")
 	time.Sleep(requestDrainTime)
 
 	// TODO: Change this to GracefulStop when integration tests are refactored.
 	s.server.Stop()
-	s.Resource.Stop()
 
 	if s.serverMetricsReporter != nil {
 		s.serverMetricsReporter.Stop(logger)
@@ -454,3 +329,7 @@ func numFrontendHosts(
 	}
 	return ringSize
 }
+
+func (s *Service) GetFaultInjection() *client.FaultInjectionDataStoreFactory {
+	return s.faultInjectionDataStoreFactory
+}
