@@ -25,6 +25,7 @@
 package history
 
 import (
+	"math/rand"
 	"testing"
 	"time"
 
@@ -40,9 +41,9 @@ import (
 
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/dynamicconfig"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/payloads"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/common/searchattribute"
@@ -50,18 +51,39 @@ import (
 	"go.temporal.io/server/service/history/tests"
 )
 
+var (
+	nonTerminalCommands = []*commandpb.Command{
+		{CommandType: enumspb.COMMAND_TYPE_SCHEDULE_ACTIVITY_TASK},
+		{CommandType: enumspb.COMMAND_TYPE_REQUEST_CANCEL_ACTIVITY_TASK},
+		{CommandType: enumspb.COMMAND_TYPE_CANCEL_TIMER},
+		{CommandType: enumspb.COMMAND_TYPE_CANCEL_TIMER},
+		{CommandType: enumspb.COMMAND_TYPE_RECORD_MARKER},
+		{CommandType: enumspb.COMMAND_TYPE_REQUEST_CANCEL_EXTERNAL_WORKFLOW_EXECUTION},
+		{CommandType: enumspb.COMMAND_TYPE_SIGNAL_EXTERNAL_WORKFLOW_EXECUTION},
+		{CommandType: enumspb.COMMAND_TYPE_START_CHILD_WORKFLOW_EXECUTION},
+		{CommandType: enumspb.COMMAND_TYPE_UPSERT_WORKFLOW_SEARCH_ATTRIBUTES},
+	}
+
+	terminalCommands = []*commandpb.Command{
+		{CommandType: enumspb.COMMAND_TYPE_CONTINUE_AS_NEW_WORKFLOW_EXECUTION},
+		{CommandType: enumspb.COMMAND_TYPE_COMPLETE_WORKFLOW_EXECUTION},
+		{CommandType: enumspb.COMMAND_TYPE_FAIL_WORKFLOW_EXECUTION},
+		{CommandType: enumspb.COMMAND_TYPE_CANCEL_WORKFLOW_EXECUTION},
+	}
+)
+
 type (
 	commandAttrValidatorSuite struct {
 		suite.Suite
 		*require.Assertions
 
 		controller         *gomock.Controller
-		mockNamespaceCache *cache.MockNamespaceCache
+		mockNamespaceCache *namespace.MockRegistry
 
 		validator *commandAttrValidator
 
-		testNamespaceID       string
-		testTargetNamespaceID string
+		testNamespaceID       namespace.ID
+		testTargetNamespaceID namespace.ID
 	}
 )
 
@@ -82,7 +104,7 @@ func (s *commandAttrValidatorSuite) SetupTest() {
 	s.Assertions = require.New(s.T())
 
 	s.controller = gomock.NewController(s.T())
-	s.mockNamespaceCache = cache.NewMockNamespaceCache(s.controller)
+	s.mockNamespaceCache = namespace.NewMockRegistry(s.controller)
 	config := &configs.Config{
 		MaxIDLengthLimit:                  dynamicconfig.GetIntPropertyFn(1000),
 		SearchAttributesNumberOfKeysLimit: dynamicconfig.GetIntPropertyFilteredByNamespace(100),
@@ -109,17 +131,15 @@ func (s *commandAttrValidatorSuite) TearDownTest() {
 }
 
 func (s *commandAttrValidatorSuite) TestValidateSignalExternalWorkflowExecutionAttributes() {
-	namespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewLocalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		cluster.TestCurrentClusterName,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewLocalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		cluster.TestCurrentClusterName,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil).AnyTimes()
@@ -154,7 +174,7 @@ func (s *commandAttrValidatorSuite) TestValidateSignalExternalWorkflowExecutionA
 }
 
 func (s *commandAttrValidatorSuite) TestValidateUpsertWorkflowSearchAttributes() {
-	namespace := "tests.Namespace"
+	namespace := namespace.Name("tests.Namespace")
 	var attributes *commandpb.UpsertWorkflowSearchAttributesCommandAttributes
 
 	err := s.validator.validateUpsertWorkflowSearchAttributes(namespace, attributes, "index-name")
@@ -178,17 +198,15 @@ func (s *commandAttrValidatorSuite) TestValidateUpsertWorkflowSearchAttributes()
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_LocalToLocal() {
-	namespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewLocalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		cluster.TestCurrentClusterName,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewLocalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		cluster.TestCurrentClusterName,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -199,21 +217,19 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_LocalToLocal(
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_LocalToEffectiveLocal_SameCluster() {
-	namespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewLocalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		cluster.TestCurrentClusterName,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
 			Clusters:          []string{cluster.TestCurrentClusterName},
 		},
 		1234,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -224,21 +240,19 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_LocalToEffect
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_LocalToEffectiveLocal_DiffCluster() {
-	namespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewLocalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		cluster.TestCurrentClusterName,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestAlternativeClusterName,
 			Clusters:          []string{cluster.TestAlternativeClusterName},
 		},
 		1234,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -249,14 +263,13 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_LocalToEffect
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_LocalToGlobal() {
-	namespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewLocalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		cluster.TestCurrentClusterName,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -266,7 +279,6 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_LocalToGlobal
 			},
 		},
 		1234,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -277,21 +289,19 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_LocalToGlobal
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLocalToLocal_SameCluster() {
-	namespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
 			Clusters:          []string{cluster.TestCurrentClusterName},
 		},
 		1234,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewLocalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		cluster.TestCurrentClusterName,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -302,21 +312,19 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLoca
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLocalToLocal_DiffCluster() {
-	namespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestAlternativeClusterName,
 			Clusters:          []string{cluster.TestAlternativeClusterName},
 		},
 		1234,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewLocalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		cluster.TestCurrentClusterName,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -327,25 +335,23 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLoca
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLocalToEffectiveLocal_SameCluster() {
-	namespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
 			Clusters:          []string{cluster.TestCurrentClusterName},
 		},
 		1234,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
 			Clusters:          []string{cluster.TestCurrentClusterName},
 		},
 		5678,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -356,25 +362,23 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLoca
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLocalToEffectiveLocal_DiffCluster() {
-	namespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
 			Clusters:          []string{cluster.TestCurrentClusterName},
 		},
 		1234,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestAlternativeClusterName,
 			Clusters:          []string{cluster.TestAlternativeClusterName},
 		},
 		5678,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -385,8 +389,8 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLoca
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLocalToGlobal() {
-	namespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -395,10 +399,9 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLoca
 			},
 		},
 		5678,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -408,7 +411,6 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLoca
 			},
 		},
 		1234,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -419,8 +421,8 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_EffectiveLoca
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_GlobalToLocal() {
-	namespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -430,13 +432,11 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_GlobalToLocal
 			},
 		},
 		1234,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewLocalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		cluster.TestCurrentClusterName,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -447,8 +447,8 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_GlobalToLocal
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_GlobalToEffectiveLocal() {
-	namespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -458,10 +458,9 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_GlobalToEffec
 			},
 		},
 		5678,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -470,7 +469,6 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_GlobalToEffec
 			},
 		},
 		1234,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -481,8 +479,8 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_GlobalToEffec
 }
 
 func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_GlobalToGlobal_DiffNamespace() {
-	namespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testNamespaceID},
+	namespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -492,10 +490,9 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_GlobalToGloba
 			},
 		},
 		1234,
-		nil,
 	)
-	targetNamespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID},
+	targetNamespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Name: s.testTargetNamespaceID.String()},
 		nil,
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -505,7 +502,6 @@ func (s *commandAttrValidatorSuite) TestValidateCrossNamespaceCall_GlobalToGloba
 			},
 		},
 		1234,
-		nil,
 	)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.testNamespaceID).Return(namespaceEntry, nil)
@@ -656,3 +652,26 @@ func (s *commandAttrValidatorSuite) TestValidateActivityRetryPolicy() {
 		})
 	}
 }
+
+func (s *commandAttrValidatorSuite) TestValidateCommandSequence_NoTerminalCommand() {
+	err := s.validator.validateCommandSequence(nonTerminalCommands)
+	s.NoError(err)
+}
+
+func (s *commandAttrValidatorSuite) TestValidateCommandSequence_ValidTerminalCommand() {
+	for _, terminalCommand := range terminalCommands {
+		err := s.validator.validateCommandSequence(append(nonTerminalCommands, terminalCommand))
+		s.NoError(err)
+	}
+}
+
+func (s *commandAttrValidatorSuite) TestValidateCommandSequence_InvalidTerminalCommand() {
+	for _, terminalCommand := range terminalCommands {
+		err := s.validator.validateCommandSequence(append(
+			[]*commandpb.Command{terminalCommand},
+			nonTerminalCommands[int(rand.Int31n(int32(len(nonTerminalCommands))))],
+		))
+		s.Error(err)
+		s.IsType(&serviceerror.InvalidArgument{}, err)
+	}
+}
