@@ -31,7 +31,8 @@ import (
 
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/dynamicconfig"
-	"go.temporal.io/server/common/task"
+	"go.temporal.io/server/common/namespace"
+	"go.temporal.io/server/common/persistence/visibility"
 )
 
 // Config represents configuration for history service
@@ -42,19 +43,22 @@ type Config struct {
 	// TODO remove this dynamic flag in 1.14.x
 	EnableDBRecordVersion dynamicconfig.BoolPropertyFn
 
-	RPS                           dynamicconfig.IntPropertyFn
-	MaxIDLengthLimit              dynamicconfig.IntPropertyFn
-	PersistenceMaxQPS             dynamicconfig.IntPropertyFn
-	PersistenceGlobalMaxQPS       dynamicconfig.IntPropertyFn
-	EnableVisibilitySampling      dynamicconfig.BoolPropertyFn
-	VisibilityOpenMaxQPS          dynamicconfig.IntPropertyFnWithNamespaceFilter
-	VisibilityClosedMaxQPS        dynamicconfig.IntPropertyFnWithNamespaceFilter
-	AdvancedVisibilityWritingMode dynamicconfig.StringPropertyFn
-	EmitShardDiffLog              dynamicconfig.BoolPropertyFn
-	MaxAutoResetPoints            dynamicconfig.IntPropertyFnWithNamespaceFilter
-	ThrottledLogRPS               dynamicconfig.IntPropertyFn
-	EnableStickyQuery             dynamicconfig.BoolPropertyFnWithNamespaceFilter
-	ShutdownDrainDuration         dynamicconfig.DurationPropertyFn
+	RPS                     dynamicconfig.IntPropertyFn
+	MaxIDLengthLimit        dynamicconfig.IntPropertyFn
+	PersistenceMaxQPS       dynamicconfig.IntPropertyFn
+	PersistenceGlobalMaxQPS dynamicconfig.IntPropertyFn
+
+	StandardVisibilityPersistenceMaxReadQPS  dynamicconfig.IntPropertyFn
+	StandardVisibilityPersistenceMaxWriteQPS dynamicconfig.IntPropertyFn
+	AdvancedVisibilityPersistenceMaxReadQPS  dynamicconfig.IntPropertyFn
+	AdvancedVisibilityPersistenceMaxWriteQPS dynamicconfig.IntPropertyFn
+	AdvancedVisibilityWritingMode            dynamicconfig.StringPropertyFn
+
+	EmitShardDiffLog      dynamicconfig.BoolPropertyFn
+	MaxAutoResetPoints    dynamicconfig.IntPropertyFnWithNamespaceFilter
+	ThrottledLogRPS       dynamicconfig.IntPropertyFn
+	EnableStickyQuery     dynamicconfig.BoolPropertyFnWithNamespaceFilter
+	ShutdownDrainDuration dynamicconfig.DurationPropertyFn
 
 	// HistoryCache settings
 	// Change of these configs require shard restart
@@ -78,14 +82,6 @@ type Config struct {
 	StandbyTaskMissingEventsResendDelay  dynamicconfig.DurationPropertyFn
 	StandbyTaskMissingEventsDiscardDelay dynamicconfig.DurationPropertyFn
 
-	// Task process settings
-	TaskProcessRPS                 dynamicconfig.IntPropertyFnWithNamespaceFilter
-	EnablePriorityTaskProcessor    dynamicconfig.BoolPropertyFn
-	TaskSchedulerType              dynamicconfig.IntPropertyFn
-	TaskSchedulerWorkerCount       dynamicconfig.IntPropertyFn
-	TaskSchedulerQueueSize         dynamicconfig.IntPropertyFn
-	TaskSchedulerRoundRobinWeights dynamicconfig.MapPropertyFn
-
 	// TimerQueueProcessor settings
 	TimerTaskBatchSize                                dynamicconfig.IntPropertyFn
 	TimerTaskWorkerCount                              dynamicconfig.IntPropertyFn
@@ -225,8 +221,7 @@ type Config struct {
 	ReplicationEventsFromCurrentCluster    dynamicconfig.BoolPropertyFnWithNamespaceFilter
 	StandbyTaskReReplicationContextTimeout dynamicconfig.DurationPropertyFnWithNamespaceIDFilter
 
-	EnableDropStuckTaskByNamespaceID dynamicconfig.BoolPropertyFnWithNamespaceIDFilter
-	SkipReapplicationByNamespaceID   dynamicconfig.BoolPropertyFnWithNamespaceIDFilter
+	SkipReapplicationByNamespaceID dynamicconfig.BoolPropertyFnWithNamespaceIDFilter
 
 	// ===== Visibility related =====
 	// VisibilityQueueProcessor settings
@@ -247,10 +242,27 @@ type Config struct {
 	VisibilityProcessorEnablePriorityTaskProcessor         dynamicconfig.BoolPropertyFn
 	VisibilityProcessorVisibilityArchivalTimeLimit         dynamicconfig.DurationPropertyFn
 
+	// TieredStorageQueueProcessor settings
+	TieredStorageTaskBatchSize                                dynamicconfig.IntPropertyFn
+	TieredStorageTaskWorkerCount                              dynamicconfig.IntPropertyFn
+	TieredStorageTaskMaxRetryCount                            dynamicconfig.IntPropertyFn
+	TieredStorageProcessorCompleteTaskFailureRetryCount       dynamicconfig.IntPropertyFn
+	TieredStorageProcessorFailoverMaxPollRPS                  dynamicconfig.IntPropertyFn
+	TieredStorageProcessorMaxPollRPS                          dynamicconfig.IntPropertyFn
+	TieredStorageProcessorMaxPollInterval                     dynamicconfig.DurationPropertyFn
+	TieredStorageProcessorMaxPollIntervalJitterCoefficient    dynamicconfig.FloatPropertyFn
+	TieredStorageProcessorUpdateAckInterval                   dynamicconfig.DurationPropertyFn
+	TieredStorageProcessorUpdateAckIntervalJitterCoefficient  dynamicconfig.FloatPropertyFn
+	TieredStorageProcessorCompleteTaskInterval                dynamicconfig.DurationPropertyFn
+	TieredStorageProcessorRedispatchInterval                  dynamicconfig.DurationPropertyFn
+	TieredStorageProcessorRedispatchIntervalJitterCoefficient dynamicconfig.FloatPropertyFn
+	TieredStorageProcessorMaxRedispatchQueueSize              dynamicconfig.IntPropertyFn
+	TieredStorageProcessorEnablePriorityTaskProcessor         dynamicconfig.BoolPropertyFn
+	TieredStorageProcessorArchivalTimeLimit                   dynamicconfig.DurationPropertyFn
+
 	SearchAttributesNumberOfKeysLimit dynamicconfig.IntPropertyFnWithNamespaceFilter
 	SearchAttributesSizeOfValueLimit  dynamicconfig.IntPropertyFnWithNamespaceFilter
 	SearchAttributesTotalSizeLimit    dynamicconfig.IntPropertyFnWithNamespaceFilter
-	ESVisibilityListMaxQPS            dynamicconfig.IntPropertyFnWithNamespaceFilter
 	IndexerConcurrency                dynamicconfig.IntPropertyFn
 	ESProcessorNumOfWorkers           dynamicconfig.IntPropertyFn
 	ESProcessorBulkActions            dynamicconfig.IntPropertyFn // max number of requests in bulk
@@ -266,25 +278,25 @@ const (
 )
 
 // NewConfig returns new service config with default values
-func NewConfig(dc *dynamicconfig.Collection, numberOfShards int32, isAdvancedVisConfigExist bool, defaultVisibilityIndex string) *Config {
+func NewConfig(dc *dynamicconfig.Collection, numberOfShards int32, isAdvancedVisibilityConfigExist bool, defaultVisibilityIndex string) *Config {
 	cfg := &Config{
 		NumberOfShards:             numberOfShards,
 		DefaultVisibilityIndexName: defaultVisibilityIndex,
 
-		// TODO remove this dynamic flag in 1.14.x
-		EnableDBRecordVersion: dc.GetBoolProperty(dynamicconfig.EnableDBRecordVersion, true),
+		RPS:                        dc.GetIntProperty(dynamicconfig.HistoryRPS, 3000),
+		MaxIDLengthLimit:           dc.GetIntProperty(dynamicconfig.MaxIDLengthLimit, 1000),
+		PersistenceMaxQPS:          dc.GetIntProperty(dynamicconfig.HistoryPersistenceMaxQPS, 9000),
+		PersistenceGlobalMaxQPS:    dc.GetIntProperty(dynamicconfig.HistoryPersistenceGlobalMaxQPS, 0),
+		ShutdownDrainDuration:      dc.GetDurationProperty(dynamicconfig.HistoryShutdownDrainDuration, 0),
+		MaxAutoResetPoints:         dc.GetIntPropertyFilteredByNamespace(dynamicconfig.HistoryMaxAutoResetPoints, DefaultHistoryMaxAutoResetPoints),
+		DefaultWorkflowTaskTimeout: dc.GetDurationPropertyFilteredByNamespace(dynamicconfig.DefaultWorkflowTaskTimeout, common.DefaultWorkflowTaskTimeout),
+
+		StandardVisibilityPersistenceMaxReadQPS:  dc.GetIntProperty(dynamicconfig.StandardVisibilityPersistenceMaxReadQPS, 9000),
+		StandardVisibilityPersistenceMaxWriteQPS: dc.GetIntProperty(dynamicconfig.StandardVisibilityPersistenceMaxWriteQPS, 9000),
+		AdvancedVisibilityPersistenceMaxReadQPS:  dc.GetIntProperty(dynamicconfig.AdvancedVisibilityPersistenceMaxReadQPS, 9000),
+		AdvancedVisibilityPersistenceMaxWriteQPS: dc.GetIntProperty(dynamicconfig.AdvancedVisibilityPersistenceMaxWriteQPS, 9000),
+		AdvancedVisibilityWritingMode:            dc.GetStringProperty(dynamicconfig.AdvancedVisibilityWritingMode, visibility.DefaultAdvancedVisibilityWritingMode(isAdvancedVisibilityConfigExist)),
 
-		RPS:                                  dc.GetIntProperty(dynamicconfig.HistoryRPS, 3000),
-		MaxIDLengthLimit:                     dc.GetIntProperty(dynamicconfig.MaxIDLengthLimit, 1000),
-		PersistenceMaxQPS:                    dc.GetIntProperty(dynamicconfig.HistoryPersistenceMaxQPS, 9000),
-		PersistenceGlobalMaxQPS:              dc.GetIntProperty(dynamicconfig.HistoryPersistenceGlobalMaxQPS, 0),
-		ShutdownDrainDuration:                dc.GetDurationProperty(dynamicconfig.HistoryShutdownDrainDuration, 0),
-		EnableVisibilitySampling:             dc.GetBoolProperty(dynamicconfig.EnableVisibilitySampling, true),
-		VisibilityOpenMaxQPS:                 dc.GetIntPropertyFilteredByNamespace(dynamicconfig.HistoryVisibilityOpenMaxQPS, 300),
-		VisibilityClosedMaxQPS:               dc.GetIntPropertyFilteredByNamespace(dynamicconfig.HistoryVisibilityClosedMaxQPS, 300),
-		MaxAutoResetPoints:                   dc.GetIntPropertyFilteredByNamespace(dynamicconfig.HistoryMaxAutoResetPoints, DefaultHistoryMaxAutoResetPoints),
-		DefaultWorkflowTaskTimeout:           dc.GetDurationPropertyFilteredByNamespace(dynamicconfig.DefaultWorkflowTaskTimeout, common.DefaultWorkflowTaskTimeout),
-		AdvancedVisibilityWritingMode:        dc.GetStringProperty(dynamicconfig.AdvancedVisibilityWritingMode, common.GetDefaultAdvancedVisibilityWritingMode(isAdvancedVisConfigExist)),
 		EmitShardDiffLog:                     dc.GetBoolProperty(dynamicconfig.EmitShardDiffLog, false),
 		HistoryCacheInitialSize:              dc.GetIntProperty(dynamicconfig.HistoryCacheInitialSize, 128),
 		HistoryCacheMaxSize:                  dc.GetIntProperty(dynamicconfig.HistoryCacheMaxSize, 512),
@@ -299,14 +311,6 @@ func NewConfig(dc *dynamicconfig.Collection, numberOfShards int32, isAdvancedVis
 		StandbyTaskMissingEventsResendDelay:  dc.GetDurationProperty(dynamicconfig.StandbyTaskMissingEventsResendDelay, 10*time.Minute),
 		StandbyTaskMissingEventsDiscardDelay: dc.GetDurationProperty(dynamicconfig.StandbyTaskMissingEventsDiscardDelay, 15*time.Minute),
 
-		TaskProcessRPS: dc.GetIntPropertyFilteredByNamespace(dynamicconfig.TaskProcessRPS, 1000),
-
-		EnablePriorityTaskProcessor:    dc.GetBoolProperty(dynamicconfig.EnablePriorityTaskProcessor, false),
-		TaskSchedulerType:              dc.GetIntProperty(dynamicconfig.TaskSchedulerType, int(task.SchedulerTypeWRR)),
-		TaskSchedulerWorkerCount:       dc.GetIntProperty(dynamicconfig.TaskSchedulerWorkerCount, 20),
-		TaskSchedulerQueueSize:         dc.GetIntProperty(dynamicconfig.TaskSchedulerQueueSize, 2000),
-		TaskSchedulerRoundRobinWeights: dc.GetMapProperty(dynamicconfig.TaskSchedulerRoundRobinWeights, ConvertWeightsToDynamicConfigValue(DefaultTaskPriorityWeight)),
-
 		TimerTaskBatchSize:                                dc.GetIntProperty(dynamicconfig.TimerTaskBatchSize, 100),
 		TimerTaskWorkerCount:                              dc.GetIntProperty(dynamicconfig.TimerTaskWorkerCount, 10),
 		TimerTaskMaxRetryCount:                            dc.GetIntProperty(dynamicconfig.TimerTaskMaxRetryCount, 100),
@@ -416,8 +420,7 @@ func NewConfig(dc *dynamicconfig.Collection, numberOfShards int32, isAdvancedVis
 		ReplicationEventsFromCurrentCluster:    dc.GetBoolPropertyFnWithNamespaceFilter(dynamicconfig.ReplicationEventsFromCurrentCluster, false),
 		StandbyTaskReReplicationContextTimeout: dc.GetDurationPropertyFilteredByNamespaceID(dynamicconfig.StandbyTaskReReplicationContextTimeout, 3*time.Minute),
 
-		EnableDropStuckTaskByNamespaceID: dc.GetBoolPropertyFnWithNamespaceIDFilter(dynamicconfig.EnableDropStuckTaskByNamespaceID, false),
-		SkipReapplicationByNamespaceID:   dc.GetBoolPropertyFnWithNamespaceIDFilter(dynamicconfig.SkipReapplicationByNamespaceID, false),
+		SkipReapplicationByNamespaceID: dc.GetBoolPropertyFnWithNamespaceIDFilter(dynamicconfig.SkipReapplicationByNamespaceID, false),
 
 		// ===== Visibility related =====
 		VisibilityTaskBatchSize:                                dc.GetIntProperty(dynamicconfig.VisibilityTaskBatchSize, 100),
@@ -437,10 +440,27 @@ func NewConfig(dc *dynamicconfig.Collection, numberOfShards int32, isAdvancedVis
 		VisibilityProcessorEnablePriorityTaskProcessor:         dc.GetBoolProperty(dynamicconfig.VisibilityProcessorEnablePriorityTaskProcessor, false),
 		VisibilityProcessorVisibilityArchivalTimeLimit:         dc.GetDurationProperty(dynamicconfig.VisibilityProcessorVisibilityArchivalTimeLimit, 200*time.Millisecond),
 
+		// ===== Tiered storage =====
+		TieredStorageTaskBatchSize:                                dc.GetIntProperty(dynamicconfig.TieredStorageTaskBatchSize, 100),
+		TieredStorageProcessorFailoverMaxPollRPS:                  dc.GetIntProperty(dynamicconfig.TieredStorageProcessorFailoverMaxPollRPS, 1),
+		TieredStorageProcessorMaxPollRPS:                          dc.GetIntProperty(dynamicconfig.TieredStorageProcessorMaxPollRPS, 20),
+		TieredStorageTaskWorkerCount:                              dc.GetIntProperty(dynamicconfig.TieredStorageTaskWorkerCount, 10),
+		TieredStorageTaskMaxRetryCount:                            dc.GetIntProperty(dynamicconfig.TieredStorageTaskMaxRetryCount, 100),
+		TieredStorageProcessorCompleteTaskFailureRetryCount:       dc.GetIntProperty(dynamicconfig.TieredStorageProcessorCompleteTaskFailureRetryCount, 10),
+		TieredStorageProcessorMaxPollInterval:                     dc.GetDurationProperty(dynamicconfig.TieredStorageProcessorMaxPollInterval, 1*time.Minute),
+		TieredStorageProcessorMaxPollIntervalJitterCoefficient:    dc.GetFloat64Property(dynamicconfig.TieredStorageProcessorMaxPollIntervalJitterCoefficient, 0.15),
+		TieredStorageProcessorUpdateAckInterval:                   dc.GetDurationProperty(dynamicconfig.TieredStorageProcessorUpdateAckInterval, 30*time.Second),
+		TieredStorageProcessorUpdateAckIntervalJitterCoefficient:  dc.GetFloat64Property(dynamicconfig.TieredStorageProcessorUpdateAckIntervalJitterCoefficient, 0.15),
+		TieredStorageProcessorCompleteTaskInterval:                dc.GetDurationProperty(dynamicconfig.TieredStorageProcessorCompleteTaskInterval, 60*time.Second),
+		TieredStorageProcessorRedispatchInterval:                  dc.GetDurationProperty(dynamicconfig.TieredStorageProcessorRedispatchInterval, 5*time.Second),
+		TieredStorageProcessorRedispatchIntervalJitterCoefficient: dc.GetFloat64Property(dynamicconfig.TieredStorageProcessorRedispatchIntervalJitterCoefficient, 0.15),
+		TieredStorageProcessorMaxRedispatchQueueSize:              dc.GetIntProperty(dynamicconfig.TieredStorageProcessorMaxRedispatchQueueSize, 10000),
+		TieredStorageProcessorEnablePriorityTaskProcessor:         dc.GetBoolProperty(dynamicconfig.TieredStorageProcessorEnablePriorityTaskProcessor, false),
+		TieredStorageProcessorArchivalTimeLimit:                   dc.GetDurationProperty(dynamicconfig.TieredStorageProcessorArchivalTimeLimit, 200*time.Millisecond),
+
 		SearchAttributesNumberOfKeysLimit: dc.GetIntPropertyFilteredByNamespace(dynamicconfig.SearchAttributesNumberOfKeysLimit, 100),
 		SearchAttributesSizeOfValueLimit:  dc.GetIntPropertyFilteredByNamespace(dynamicconfig.SearchAttributesSizeOfValueLimit, 2*1024),
 		SearchAttributesTotalSizeLimit:    dc.GetIntPropertyFilteredByNamespace(dynamicconfig.SearchAttributesTotalSizeLimit, 40*1024),
-		ESVisibilityListMaxQPS:            dc.GetIntPropertyFilteredByNamespace(dynamicconfig.FrontendESVisibilityListMaxQPS, 10),
 		IndexerConcurrency:                dc.GetIntProperty(dynamicconfig.WorkerIndexerConcurrency, 100),
 		ESProcessorNumOfWorkers:           dc.GetIntProperty(dynamicconfig.WorkerESProcessorNumOfWorkers, 1),
 		// Should be not greater than NumberOfShards(512)/NumberOfHistoryNodes(4) * VisibilityTaskWorkerCount(10)/ESProcessorNumOfWorkers(1) divided by workflow distribution factor (2 at least).
@@ -459,8 +479,8 @@ func NewConfig(dc *dynamicconfig.Collection, numberOfShards int32, isAdvancedVis
 }
 
 // GetShardID return the corresponding shard ID for a given namespaceID and workflowID pair
-func (config *Config) GetShardID(namespaceID, workflowID string) int32 {
-	return common.WorkflowIDToHistoryShard(namespaceID, workflowID, config.NumberOfShards)
+func (config *Config) GetShardID(namespaceID namespace.ID, workflowID string) int32 {
+	return common.WorkflowIDToHistoryShard(namespaceID.String(), workflowID, config.NumberOfShards)
 }
 
 func NewDynamicConfig() *Config {
