@@ -37,11 +37,11 @@ import (
 
 	"go.temporal.io/server/api/historyservice/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/persistence/versionhistory"
 	"go.temporal.io/server/common/primitives/timestamp"
@@ -67,7 +67,7 @@ type (
 	) workflow.MutableStateRebuilder
 
 	mutableStateProvider func(
-		namespaceEntry *cache.NamespaceCacheEntry,
+		namespaceEntry *namespace.Namespace,
 		startTime time.Time,
 		logger log.Logger,
 	) workflow.MutableState
@@ -85,7 +85,7 @@ type (
 	) nDCConflictResolver
 
 	nDCWorkflowResetterProvider func(
-		namespaceID string,
+		namespaceID namespace.ID,
 		workflowID string,
 		baseRunID string,
 		newContext workflow.Context,
@@ -106,8 +106,8 @@ type (
 		executionMgr      persistence.ExecutionManager
 		historySerializer serialization.Serializer
 		metricsClient     metrics.Client
-		namespaceCache    cache.NamespaceCache
-		historyCache      *workflow.Cache
+		namespaceRegistry namespace.Registry
+		historyCache      workflow.Cache
 		eventsReapplier   nDCEventsReapplier
 		transactionMgr    nDCTransactionMgr
 		logger            log.Logger
@@ -120,11 +120,11 @@ type (
 	}
 )
 
-var errPanic = serviceerror.NewInternal("encounter panic")
+var errPanic = serviceerror.NewInternal("encountered panic")
 
 func newNDCHistoryReplicator(
 	shard shard.Context,
-	historyCache *workflow.Cache,
+	historyCache workflow.Cache,
 	eventsReapplier nDCEventsReapplier,
 	logger log.Logger,
 ) *nDCHistoryReplicatorImpl {
@@ -132,11 +132,11 @@ func newNDCHistoryReplicator(
 	transactionMgr := newNDCTransactionMgr(shard, historyCache, eventsReapplier, logger)
 	replicator := &nDCHistoryReplicatorImpl{
 		shard:             shard,
-		clusterMetadata:   shard.GetService().GetClusterMetadata(),
+		clusterMetadata:   shard.GetClusterMetadata(),
 		executionMgr:      shard.GetExecutionManager(),
 		historySerializer: serialization.NewSerializer(),
 		metricsClient:     shard.GetMetricsClient(),
-		namespaceCache:    shard.GetNamespaceCache(),
+		namespaceRegistry: shard.GetNamespaceRegistry(),
 		historyCache:      historyCache,
 		transactionMgr:    transactionMgr,
 		eventsReapplier:   eventsReapplier,
@@ -157,7 +157,7 @@ func newNDCHistoryReplicator(
 			return newNDCConflictResolver(shard, context, mutableState, logger)
 		},
 		newWorkflowResetter: func(
-			namespaceID string,
+			namespaceID namespace.ID,
 			workflowID string,
 			baseRunID string,
 			newContext workflow.Context,
@@ -176,12 +176,12 @@ func newNDCHistoryReplicator(
 				logger,
 				state,
 				func(mutableState workflow.MutableState) workflow.TaskGenerator {
-					return workflow.NewTaskGenerator(shard.GetNamespaceCache(), logger, mutableState)
+					return workflow.NewTaskGenerator(shard.GetNamespaceRegistry(), logger, mutableState)
 				},
 			)
 		},
 		newMutableState: func(
-			namespaceEntry *cache.NamespaceCacheEntry,
+			namespaceEntry *namespace.Namespace,
 			startTime time.Time,
 			logger log.Logger,
 		) workflow.MutableState {
@@ -252,12 +252,12 @@ func (r *nDCHistoryReplicatorImpl) applyEvents(
 		// the continue as new + start workflow execution combination will also be processed here
 		var mutableState workflow.MutableState
 		var err error
-		namespaceEntry, err := r.namespaceCache.GetNamespaceByID(context.GetNamespaceID())
+		namespaceEntry, err := r.namespaceRegistry.GetNamespaceByID(context.GetNamespaceID())
 		if err != nil {
 			return err
 		}
 
-		if r.shard.GetConfig().ReplicationEventsFromCurrentCluster(namespaceEntry.GetInfo().Name) {
+		if r.shard.GetConfig().ReplicationEventsFromCurrentCluster(namespaceEntry.Name().String()) {
 			// this branch is used when replicating events (generated from current cluster)from remote cluster to current cluster.
 			// this could happen when the events are lost in current cluster and plan to recover them from remote cluster.
 			mutableState, err = context.LoadWorkflowExecutionForReplication(task.getVersion())
@@ -312,7 +312,7 @@ func (r *nDCHistoryReplicatorImpl) applyStartEvents(
 	task nDCReplicationTask,
 ) (retError error) {
 
-	namespaceEntry, err := r.namespaceCache.GetNamespaceByID(task.getNamespaceID())
+	namespaceEntry, err := r.namespaceRegistry.GetNamespaceByID(task.getNamespaceID())
 	if err != nil {
 		return err
 	}
@@ -341,7 +341,7 @@ func (r *nDCHistoryReplicatorImpl) applyStartEvents(
 		task.getEventTime(),
 		newNDCWorkflow(
 			ctx,
-			r.namespaceCache,
+			r.namespaceRegistry,
 			r.clusterMetadata,
 			context,
 			mutableState,
@@ -442,7 +442,7 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToCurrentBranch(
 
 	targetWorkflow := newNDCWorkflow(
 		ctx,
-		r.namespaceCache,
+		r.namespaceRegistry,
 		r.clusterMetadata,
 		context,
 		mutableState,
@@ -454,7 +454,7 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToCurrentBranch(
 		newExecutionInfo := newMutableState.GetExecutionInfo()
 		newExecutionState := newMutableState.GetExecutionState()
 		newContext := workflow.NewContext(
-			newExecutionInfo.NamespaceId,
+			namespace.ID(newExecutionInfo.NamespaceId),
 			commonpb.WorkflowExecution{
 				WorkflowId: newExecutionInfo.WorkflowId,
 				RunId:      newExecutionState.RunId,
@@ -465,7 +465,7 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToCurrentBranch(
 
 		newWorkflow = newNDCWorkflow(
 			ctx,
-			r.namespaceCache,
+			r.namespaceRegistry,
 			r.clusterMetadata,
 			newContext,
 			newMutableState,
@@ -550,14 +550,14 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToNoneCurrentBranchWithout
 		task.getEventTime(),
 		newNDCWorkflow(
 			ctx,
-			r.namespaceCache,
+			r.namespaceRegistry,
 			r.clusterMetadata,
 			context,
 			mutableState,
 			releaseFn,
 		),
 		&persistence.WorkflowEvents{
-			NamespaceID: task.getNamespaceID(),
+			NamespaceID: task.getNamespaceID().String(),
 			WorkflowID:  task.getExecution().GetWorkflowId(),
 			RunID:       task.getExecution().GetRunId(),
 			BranchToken: versionHistory.GetBranchToken(),
@@ -632,7 +632,7 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsMissingMutableState(
 		firstEvent := task.getFirstEvent()
 		return nil, serviceerrors.NewRetryReplication(
 			mutableStateMissingMessage,
-			task.getNamespaceID(),
+			task.getNamespaceID().String(),
 			task.getWorkflowID(),
 			task.getRunID(),
 			common.EmptyEventID,
@@ -702,7 +702,7 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsResetWorkflow(
 
 	targetWorkflow := newNDCWorkflow(
 		ctx,
-		r.namespaceCache,
+		r.namespaceRegistry,
 		r.clusterMetadata,
 		context,
 		mutableState,
