@@ -34,8 +34,8 @@ import (
 	"go.temporal.io/api/serviceerror"
 
 	enumsspb "go.temporal.io/server/api/enums/v1"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/payloads"
 	"go.temporal.io/server/service/history/consts"
 	"go.temporal.io/server/service/history/workflow"
@@ -54,8 +54,8 @@ type (
 	}
 
 	nDCWorkflowImpl struct {
-		namespaceCache  cache.NamespaceCache
-		clusterMetadata cluster.Metadata
+		namespaceRegistry namespace.Registry
+		clusterMetadata   cluster.Metadata
 
 		ctx          context.Context
 		context      workflow.Context
@@ -66,7 +66,7 @@ type (
 
 func newNDCWorkflow(
 	ctx context.Context,
-	namespaceCache cache.NamespaceCache,
+	namespaceRegistry namespace.Registry,
 	clusterMetadata cluster.Metadata,
 	context workflow.Context,
 	mutableState workflow.MutableState,
@@ -74,9 +74,9 @@ func newNDCWorkflow(
 ) *nDCWorkflowImpl {
 
 	return &nDCWorkflowImpl{
-		ctx:             ctx,
-		namespaceCache:  namespaceCache,
-		clusterMetadata: clusterMetadata,
+		ctx:               ctx,
+		namespaceRegistry: namespaceRegistry,
+		clusterMetadata:   clusterMetadata,
 
 		context:      context,
 		mutableState: mutableState,
@@ -211,7 +211,7 @@ func (r *nDCWorkflowImpl) flushBufferedEvents() error {
 	currentCluster := r.clusterMetadata.GetCurrentClusterName()
 
 	if lastWriteCluster != currentCluster {
-		return serviceerror.NewInternal("nDCWorkflow encounter workflow with buffered events but last write not from current cluster")
+		return serviceerror.NewInternal("nDCWorkflow encountered workflow with buffered events but last write not from current cluster")
 	}
 
 	return r.failWorkflowTask(lastWriteVersion)
