@@ -28,13 +28,19 @@ import (
 	"time"
 
 	commonpb "go.temporal.io/api/common/v1"
-	"go.temporal.io/server/common/cache"
+
+	"go.temporal.io/server/api/adminservice/v1"
+	"go.temporal.io/server/api/historyservice/v1"
+	"go.temporal.io/server/common/archiver"
 	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/cluster"
+	"go.temporal.io/server/common/definition"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
-	"go.temporal.io/server/common/resource"
+	"go.temporal.io/server/common/persistence/serialization"
+	"go.temporal.io/server/common/searchattribute"
 	"go.temporal.io/server/service/history/configs"
 	"go.temporal.io/server/service/history/events"
 )
@@ -45,9 +51,8 @@ type (
 	// Context represents a history engine shard
 	Context interface {
 		GetShardID() int32
-		GetService() resource.Resource
 		GetExecutionManager() persistence.ExecutionManager
-		GetNamespaceCache() cache.NamespaceCache
+		GetNamespaceRegistry() namespace.Registry
 		GetClusterMetadata() cluster.Metadata
 		GetConfig() *configs.Config
 		GetEventsCache() events.Cache
@@ -55,10 +60,8 @@ type (
 		GetThrottledLogger() log.Logger
 		GetMetricsClient() metrics.Client
 		GetTimeSource() clock.TimeSource
-		PreviousShardOwnerWasDifferent() bool
 
-		GetEngine() Engine
-		SetEngine(Engine)
+		GetEngine() (Engine, error)
 
 		GenerateTransferTaskID() (int64, error)
 		GenerateTransferTaskIDs(number int) ([]int64, error)
@@ -71,6 +74,8 @@ type (
 		GetLastUpdatedTime() time.Time
 		GetTimerMaxReadLevel(cluster string) time.Time
 
+		GetReplicationStatus(cluster []string) (map[string]*historyservice.ShardReplicationStatusPerCluster, map[string]*historyservice.HandoverNamespaceInfo, error)
+
 		GetTransferAckLevel() int64
 		UpdateTransferAckLevel(ackLevel int64) error
 		GetTransferClusterAckLevel(cluster string) int64
@@ -79,13 +84,16 @@ type (
 		GetVisibilityAckLevel() int64
 		UpdateVisibilityAckLevel(ackLevel int64) error
 
+		GetTieredStorageAckLevel() int64
+		UpdateTieredStorageAckLevel(ackLevel int64) error
+
 		GetReplicatorAckLevel() int64
 		UpdateReplicatorAckLevel(ackLevel int64) error
 		GetReplicatorDLQAckLevel(sourceCluster string) int64
 		UpdateReplicatorDLQAckLevel(sourCluster string, ackLevel int64) error
 
 		GetClusterReplicationLevel(cluster string) int64
-		UpdateClusterReplicationLevel(cluster string, ackTaskID int64) error
+		UpdateClusterReplicationLevel(cluster string, ackTaskID int64, ackTimestamp time.Time) error
 
 		GetTimerAckLevel() time.Time
 		UpdateTimerAckLevel(ackLevel time.Time) error
@@ -102,11 +110,23 @@ type (
 
 		GetNamespaceNotificationVersion() int64
 		UpdateNamespaceNotificationVersion(namespaceNotificationVersion int64) error
+		UpdateHandoverNamespaces(newNamespaces []*namespace.Namespace, maxRepTaskID int64)
 
 		CreateWorkflowExecution(request *persistence.CreateWorkflowExecutionRequest) (*persistence.CreateWorkflowExecutionResponse, error)
 		UpdateWorkflowExecution(request *persistence.UpdateWorkflowExecutionRequest) (*persistence.UpdateWorkflowExecutionResponse, error)
-		ConflictResolveWorkflowExecution(request *persistence.ConflictResolveWorkflowExecutionRequest) error
+		ConflictResolveWorkflowExecution(request *persistence.ConflictResolveWorkflowExecutionRequest) (*persistence.ConflictResolveWorkflowExecutionResponse, error)
+		// Delete workflow execution, current workflow execution, and add task to delete visibility.
+		// If branchToken != nil, then delete history also, otherwise leave history.
+		DeleteWorkflowExecution(workflowKey definition.WorkflowKey, branchToken []byte, version int64) error
 		AddTasks(request *persistence.AddTasksRequest) error
-		AppendHistoryEvents(request *persistence.AppendHistoryNodesRequest, namespaceID string, execution commonpb.WorkflowExecution) (int, error)
+		AppendHistoryEvents(request *persistence.AppendHistoryNodesRequest, namespaceID namespace.ID, execution commonpb.WorkflowExecution) (int, error)
+
+		GetRemoteAdminClient(cluster string) adminservice.AdminServiceClient
+		GetHistoryClient() historyservice.HistoryServiceClient
+		GetPayloadSerializer() serialization.Serializer
+
+		GetSearchAttributesProvider() searchattribute.Provider
+		GetSearchAttributesMapper() searchattribute.Mapper
+		GetArchivalMetadata() archiver.ArchivalMetadata
 	}
 )
