@@ -31,20 +31,18 @@ import (
 
 	"go.temporal.io/api/serviceerror"
 
-	enumsspb "go.temporal.io/server/api/enums/v1"
-	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/backoff"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
-	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/service/history/configs"
 	"go.temporal.io/server/service/history/consts"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 	"go.temporal.io/server/service/history/workflow"
 )
 
@@ -59,9 +57,10 @@ type (
 	}
 
 	taskInfo struct {
+		tasks.Task
+
 		// TODO: change to queueTaskExecutor
 		processor taskExecutor
-		task      queueTaskInfo
 
 		attempt   int
 		startTime time.Time
@@ -76,7 +75,7 @@ type (
 
 	taskProcessor struct {
 		shard         shard.Context
-		cache         *workflow.Cache
+		cache         workflow.Cache
 		shutdownCh    chan struct{}
 		tasksCh       chan *taskInfo
 		config        *configs.Config
@@ -95,12 +94,13 @@ type (
 
 func newTaskInfo(
 	processor taskExecutor,
-	task queueTaskInfo,
+	task tasks.Task,
 	logger log.Logger,
 ) *taskInfo {
 	return &taskInfo{
+		Task: task,
+
 		processor:         processor,
-		task:              task,
 		attempt:           1,
 		startTime:         time.Now().UTC(), // used for metrics
 		userLatency:       0,
@@ -112,13 +112,13 @@ func newTaskInfo(
 func newTaskProcessor(
 	options taskProcessorOptions,
 	shard shard.Context,
-	historyCache *workflow.Cache,
+	historyCache workflow.Cache,
 	logger log.Logger,
 ) *taskProcessor {
 
-	workerNotificationChans := []chan struct{}{}
+	var workerNotificationChs []chan struct{}
 	for index := 0; index < options.workerCount; index++ {
-		workerNotificationChans = append(workerNotificationChans, make(chan struct{}, 1))
+		workerNotificationChs = append(workerNotificationChs, make(chan struct{}, 1))
 	}
 
 	base := &taskProcessor{
@@ -130,8 +130,8 @@ func newTaskProcessor(
 		logger:                  logger,
 		metricsClient:           shard.GetMetricsClient(),
 		timeSource:              shard.GetTimeSource(),
-		workerNotificationChans: workerNotificationChans,
-		retryPolicy:             common.CreatePersistanceRetryPolicy(),
+		workerNotificationChans: workerNotificationChs,
+		retryPolicy:             common.CreatePersistenceRetryPolicy(),
 		numOfWorker:             options.workerCount,
 	}
 
@@ -209,7 +209,7 @@ FilterLoop:
 			// this must return without ack
 			return
 		default:
-			task.shouldProcessTask, err = task.processor.getTaskFilter()(task.task)
+			task.shouldProcessTask, err = task.processor.getTaskFilter()(task.Task)
 			if err == nil {
 				break FilterLoop
 			}
@@ -225,7 +225,7 @@ FilterLoop:
 			if task.attempt > t.config.TimerTaskMaxRetryCount() {
 				scope.RecordDistribution(metrics.TaskAttemptTimer, task.attempt)
 				task.logger.Error("Critical error processing task, retrying.",
-					tag.Error(err), tag.OperationCritical, tag.TaskType(task.task.GetTaskType()))
+					tag.Error(err), tag.OperationCritical, tag.Task(task.Task))
 			}
 		}
 		return err
@@ -270,7 +270,7 @@ func (t *taskProcessor) processTaskOnce(
 	if duration, ok := metrics.ContextCounterGet(ctx, metrics.HistoryWorkflowExecutionCacheLatency); ok {
 		task.userLatency += time.Duration(duration)
 	}
-	scope := t.metricsClient.Scope(scopeIdx).Tagged(t.getNamespaceTagByID(task.task.GetNamespaceId()))
+	scope := t.metricsClient.Scope(scopeIdx).Tagged(t.getNamespaceTagByID(namespace.ID(task.GetNamespaceID())))
 	if task.shouldProcessTask {
 		scope.IncCounter(metrics.TaskRequests)
 		scope.RecordTimer(metrics.TaskProcessingLatency, time.Since(startTime))
@@ -295,15 +295,6 @@ func (t *taskProcessor) handleTaskError(
 		return nil
 	}
 
-	if transferTask, ok := task.task.(*persistencespb.TransferTaskInfo); ok &&
-		transferTask.TaskType == enumsspb.TASK_TYPE_TRANSFER_CLOSE_EXECUTION &&
-		err == workflow.ErrMissingWorkflowStartEvent &&
-		t.config.EnableDropStuckTaskByNamespaceID(task.task.GetNamespaceId()) { // use namespaceID here to avoid accessing namespaceCache
-		scope.IncCounter(metrics.TransferTaskMissingEventCounter)
-		task.logger.Error("Drop close execution transfer task due to corrupted workflow history", tag.Error(err), tag.LifeCycleProcessingFailed)
-		return nil
-	}
-
 	// this is a transient error
 	if err == consts.ErrTaskRetry {
 		scope.IncCounter(metrics.TaskStandbyRetryCounter)
@@ -323,7 +314,7 @@ func (t *taskProcessor) handleTaskError(
 	// TODO remove this error check special case
 	//  since the new task life cycle will not give up until task processed / verified
 	if _, ok := err.(*serviceerror.NamespaceNotActive); ok {
-		if t.timeSource.Now().Sub(task.startTime) > 2*cache.NamespaceCacheRefreshInterval {
+		if t.timeSource.Now().Sub(task.startTime) > 2*namespace.CacheRefreshInterval {
 			scope.IncCounter(metrics.TaskNotActiveCounter)
 			return nil
 		}
@@ -348,21 +339,20 @@ func (t *taskProcessor) ackTaskOnce(
 ) {
 	task.processor.complete(task)
 	if task.shouldProcessTask {
-		goVisibilityTime := timestamp.TimeValue(task.task.GetVisibilityTime())
 		scope.RecordDistribution(metrics.TaskAttemptTimer, task.attempt)
 		scope.RecordTimer(metrics.TaskLatency, time.Since(task.startTime))
-		scope.RecordTimer(metrics.TaskQueueLatency, time.Since(goVisibilityTime))
+		scope.RecordTimer(metrics.TaskQueueLatency, time.Since(task.GetVisibilityTime()))
 		scope.RecordTimer(metrics.TaskUserLatency, task.userLatency)
 		scope.RecordTimer(metrics.TaskNoUserLatency, time.Since(task.startTime)-task.userLatency)
-		scope.RecordTimer(metrics.TaskNoUserQueueLatency, time.Since(goVisibilityTime)-task.userLatency)
+		scope.RecordTimer(metrics.TaskNoUserQueueLatency, time.Since(task.GetVisibilityTime())-task.userLatency)
 	}
 }
 
-func (t *taskProcessor) getNamespaceTagByID(namespaceID string) metrics.Tag {
-	namespace, err := t.shard.GetNamespaceCache().GetNamespaceName(namespaceID)
+func (t *taskProcessor) getNamespaceTagByID(namespaceID namespace.ID) metrics.Tag {
+	namespaceName, err := t.shard.GetNamespaceRegistry().GetNamespaceName(namespaceID)
 	if err != nil {
 		t.logger.Error("Unable to get namespace", tag.Error(err))
 		return metrics.NamespaceUnknownTag()
 	}
-	return metrics.NamespaceTag(namespace)
+	return metrics.NamespaceTag(namespaceName.String())
 }
