@@ -40,6 +40,7 @@ import (
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/service/history/configs"
 	"go.temporal.io/server/service/history/shard"
@@ -48,7 +49,22 @@ import (
 type (
 	ReleaseCacheFunc func(err error)
 
-	Cache struct {
+	Cache interface {
+		GetOrCreateCurrentWorkflowExecution(
+			ctx context.Context,
+			namespaceID namespace.ID,
+			workflowID string,
+		) (Context, ReleaseCacheFunc, error)
+
+		GetOrCreateWorkflowExecution(
+			ctx context.Context,
+			namespaceID namespace.ID,
+			execution commonpb.WorkflowExecution,
+			caller CallerType,
+		) (Context, ReleaseCacheFunc, error)
+	}
+
+	CacheImpl struct {
 		cache.Cache
 		shard            shard.Context
 		executionManager persistence.ExecutionManager
@@ -56,6 +72,8 @@ type (
 		metricsClient    metrics.Client
 		config           *configs.Config
 	}
+
+	NewCacheFn func(shard shard.Context) Cache
 )
 
 var NoopReleaseFn ReleaseCacheFunc = func(err error) {}
@@ -65,14 +83,14 @@ const (
 	cacheReleased    int32 = 1
 )
 
-func NewCache(shard shard.Context) *Cache {
+func NewCache(shard shard.Context) Cache {
 	opts := &cache.Options{}
 	config := shard.GetConfig()
 	opts.InitialCapacity = config.HistoryCacheInitialSize()
 	opts.TTL = config.HistoryCacheTTL()
 	opts.Pin = true
 
-	return &Cache{
+	return &CacheImpl{
 		Cache:            cache.New(config.HistoryCacheMaxSize(), opts),
 		shard:            shard,
 		executionManager: shard.GetExecutionManager(),
@@ -82,9 +100,9 @@ func NewCache(shard shard.Context) *Cache {
 	}
 }
 
-func (c *Cache) GetOrCreateCurrentWorkflowExecution(
+func (c *CacheImpl) GetOrCreateCurrentWorkflowExecution(
 	ctx context.Context,
-	namespaceID string,
+	namespaceID namespace.ID,
 	workflowID string,
 ) (Context, ReleaseCacheFunc, error) {
 
@@ -110,9 +128,9 @@ func (c *Cache) GetOrCreateCurrentWorkflowExecution(
 	)
 }
 
-func (c *Cache) GetOrCreateWorkflowExecution(
+func (c *CacheImpl) GetOrCreateWorkflowExecution(
 	ctx context.Context,
-	namespaceID string,
+	namespaceID namespace.ID,
 	execution commonpb.WorkflowExecution,
 	caller CallerType,
 ) (Context, ReleaseCacheFunc, error) {
@@ -140,16 +158,17 @@ func (c *Cache) GetOrCreateWorkflowExecution(
 
 	return weCtx, weReleaseFunc, err
 }
-func (c *Cache) getOrCreateWorkflowExecutionInternal(
+
+func (c *CacheImpl) getOrCreateWorkflowExecutionInternal(
 	ctx context.Context,
-	namespaceID string,
+	namespaceID namespace.ID,
 	execution commonpb.WorkflowExecution,
 	scope int,
 	forceClearContext bool,
 	caller CallerType,
 ) (Context, ReleaseCacheFunc, error) {
 
-	key := definition.NewWorkflowIdentifier(namespaceID, execution.GetWorkflowId(), execution.GetRunId())
+	key := definition.NewWorkflowKey(namespaceID.String(), execution.GetWorkflowId(), execution.GetRunId())
 	workflowCtx, cacheHit := c.Get(key).(Context)
 	if !cacheHit {
 		c.metricsClient.IncCounter(scope, metrics.CacheMissCounter)
@@ -177,8 +196,8 @@ func (c *Cache) getOrCreateWorkflowExecutionInternal(
 	return workflowCtx, releaseFunc, nil
 }
 
-func (c *Cache) makeReleaseFunc(
-	key definition.WorkflowIdentifier,
+func (c *CacheImpl) makeReleaseFunc(
+	key definition.WorkflowKey,
 	context Context,
 	forceClearContext bool,
 	caller CallerType,
@@ -204,8 +223,8 @@ func (c *Cache) makeReleaseFunc(
 	}
 }
 
-func (c *Cache) validateWorkflowExecutionInfo(
-	namespaceID string,
+func (c *CacheImpl) validateWorkflowExecutionInfo(
+	namespaceID namespace.ID,
 	execution *commonpb.WorkflowExecution,
 ) error {
 
@@ -217,7 +236,7 @@ func (c *Cache) validateWorkflowExecutionInfo(
 	if execution.GetRunId() == "" {
 		response, err := c.getCurrentExecutionWithRetry(&persistence.GetCurrentExecutionRequest{
 			ShardID:     c.shard.GetShardID(),
-			NamespaceID: namespaceID,
+			NamespaceID: namespaceID.String(),
 			WorkflowID:  execution.GetWorkflowId(),
 		})
 
@@ -232,7 +251,7 @@ func (c *Cache) validateWorkflowExecutionInfo(
 	return nil
 }
 
-func (c *Cache) getCurrentExecutionWithRetry(
+func (c *CacheImpl) getCurrentExecutionWithRetry(
 	request *persistence.GetCurrentExecutionRequest,
 ) (*persistence.GetCurrentExecutionResponse, error) {
 
