@@ -26,20 +26,29 @@ package workflow
 
 import (
 	commonpb "go.temporal.io/api/common/v1"
+	enumspb "go.temporal.io/api/enums/v1"
 	"go.temporal.io/api/serviceerror"
 
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/backoff"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
+	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/persistence/versionhistory"
 	"go.temporal.io/server/service/history/consts"
 	"go.temporal.io/server/service/history/events"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 type (
+	completionMetric struct {
+		initialized bool
+		taskQueue   string
+		status      enumspb.WorkflowExecutionStatus
+	}
 	TransactionImpl struct {
 		shard  shard.Context
 		logger log.Logger
@@ -74,13 +83,16 @@ func (t *TransactionImpl) CreateWorkflowExecution(
 		return 0, err
 	}
 
-	engine := t.shard.GetEngine()
+	engine, err := t.shard.GetEngine()
+	if err != nil {
+		return 0, err
+	}
 	NotifyWorkflowSnapshotTasks(engine, newWorkflowSnapshot)
 	if err := NotifyNewHistorySnapshotEvent(engine, newWorkflowSnapshot); err != nil {
 		t.logger.Error("unable to notify workflow creation", tag.Error(err))
 	}
 
-	return resp.HistorySize, nil
+	return int64(resp.NewMutableStateStats.HistoryStatistics.SizeDiff), nil
 }
 
 func (t *TransactionImpl) ConflictResolveWorkflowExecution(
@@ -93,53 +105,25 @@ func (t *TransactionImpl) ConflictResolveWorkflowExecution(
 	currentWorkflowEventsSeq []*persistence.WorkflowEvents,
 ) (int64, int64, int64, error) {
 
-	resetHistorySizeDiff := int64(0)
-	newWorkflowHistorySizeDiff := int64(0)
-	currentWorkflowHistorySizeDiff := int64(0)
-
-	for _, workflowEvents := range resetWorkflowEventsSeq {
-		eventsSize, err := PersistWorkflowEvents(t.shard, workflowEvents)
-		if err != nil {
-			return 0, 0, 0, err
-		}
-		resetHistorySizeDiff += eventsSize
-	}
-	resetWorkflowSnapshot.ExecutionInfo.ExecutionStats.HistorySize += resetHistorySizeDiff
-
-	if newWorkflowSnapshot != nil {
-		for _, workflowEvents := range newWorkflowEventsSeq {
-			eventsSize, err := PersistWorkflowEvents(t.shard, workflowEvents)
-			if err != nil {
-				return 0, 0, 0, err
-			}
-			newWorkflowHistorySizeDiff += eventsSize
-		}
-		newWorkflowSnapshot.ExecutionInfo.ExecutionStats.HistorySize += newWorkflowHistorySizeDiff
-	}
-
-	if currentWorkflowMutation != nil {
-		for _, workflowEvents := range currentWorkflowEventsSeq {
-			eventsSize, err := PersistWorkflowEvents(t.shard, workflowEvents)
-			if err != nil {
-				return 0, 0, 0, err
-			}
-			currentWorkflowHistorySizeDiff += eventsSize
-		}
-		currentWorkflowMutation.ExecutionInfo.ExecutionStats.HistorySize += currentWorkflowHistorySizeDiff
-	}
-
-	if err := t.shard.ConflictResolveWorkflowExecution(&persistence.ConflictResolveWorkflowExecutionRequest{
+	resp, err := conflictResolveWorkflowExecutionWithRetry(t.shard, &persistence.ConflictResolveWorkflowExecutionRequest{
 		ShardID: t.shard.GetShardID(),
 		// RangeID , this is set by shard context
 		Mode:                    conflictResolveMode,
 		ResetWorkflowSnapshot:   *resetWorkflowSnapshot,
+		ResetWorkflowEvents:     resetWorkflowEventsSeq,
 		NewWorkflowSnapshot:     newWorkflowSnapshot,
+		NewWorkflowEvents:       newWorkflowEventsSeq,
 		CurrentWorkflowMutation: currentWorkflowMutation,
-	}); err != nil {
+		CurrentWorkflowEvents:   currentWorkflowEventsSeq,
+	})
+	if err != nil {
 		return 0, 0, 0, err
 	}
 
-	engine := t.shard.GetEngine()
+	engine, err := t.shard.GetEngine()
+	if err != nil {
+		return 0, 0, 0, err
+	}
 	NotifyWorkflowSnapshotTasks(engine, resetWorkflowSnapshot)
 	NotifyWorkflowSnapshotTasks(engine, newWorkflowSnapshot)
 	NotifyWorkflowMutationTasks(engine, currentWorkflowMutation)
@@ -152,8 +136,16 @@ func (t *TransactionImpl) ConflictResolveWorkflowExecution(
 	if err := NotifyNewHistoryMutationEvent(engine, currentWorkflowMutation); err != nil {
 		t.logger.Error("unable to notify workflow mutation", tag.Error(err))
 	}
-
-	return resetHistorySizeDiff, newWorkflowHistorySizeDiff, currentWorkflowHistorySizeDiff, nil
+	resetHistorySizeDiff := int64(resp.ResetMutableStateStats.HistoryStatistics.SizeDiff)
+	newHistorySizeDiff := int64(0)
+	if resp.NewMutableStateStats != nil {
+		newHistorySizeDiff = int64(resp.NewMutableStateStats.HistoryStatistics.SizeDiff)
+	}
+	currentHistorySizeDiff := int64(0)
+	if resp.CurrentMutableStateStats != nil {
+		currentHistorySizeDiff = int64(resp.CurrentMutableStateStats.HistoryStatistics.SizeDiff)
+	}
+	return resetHistorySizeDiff, newHistorySizeDiff, currentHistorySizeDiff, nil
 }
 
 func (t *TransactionImpl) UpdateWorkflowExecution(
@@ -177,7 +169,10 @@ func (t *TransactionImpl) UpdateWorkflowExecution(
 		return 0, 0, err
 	}
 
-	engine := t.shard.GetEngine()
+	engine, err := t.shard.GetEngine()
+	if err != nil {
+		return 0, 0, err
+	}
 	NotifyWorkflowMutationTasks(engine, currentWorkflowMutation)
 	NotifyWorkflowSnapshotTasks(engine, newWorkflowSnapshot)
 	if err := NotifyNewHistoryMutationEvent(engine, currentWorkflowMutation); err != nil {
@@ -186,8 +181,12 @@ func (t *TransactionImpl) UpdateWorkflowExecution(
 	if err := NotifyNewHistorySnapshotEvent(engine, newWorkflowSnapshot); err != nil {
 		t.logger.Error("unable to notify workflow creation", tag.Error(err))
 	}
-	stats := resp.MutableStateUpdateSessionStats
-	return stats.UpdateWorkflowHistorySizeDiff, stats.NewWorkflowHistorySizeDiff, nil
+	updateHistorySizeDiff := int64(resp.UpdateMutableStateStats.HistoryStatistics.SizeDiff)
+	newHistorySizeDiff := int64(0)
+	if resp.NewMutableStateStats != nil {
+		newHistorySizeDiff = int64(resp.NewMutableStateStats.HistoryStatistics.SizeDiff)
+	}
+	return updateHistorySizeDiff, newHistorySizeDiff, nil
 }
 
 func PersistWorkflowEvents(
@@ -211,7 +210,7 @@ func persistFirstWorkflowEvents(
 	workflowEvents *persistence.WorkflowEvents,
 ) (int64, error) {
 
-	namespaceID := workflowEvents.NamespaceID
+	namespaceID := namespace.ID(workflowEvents.NamespaceID)
 	workflowID := workflowEvents.WorkflowID
 	runID := workflowEvents.RunID
 	execution := commonpb.WorkflowExecution{
@@ -229,7 +228,7 @@ func persistFirstWorkflowEvents(
 		execution,
 		&persistence.AppendHistoryNodesRequest{
 			IsNewBranch:       true,
-			Info:              persistence.BuildHistoryGarbageCleanupInfo(namespaceID, workflowID, runID),
+			Info:              persistence.BuildHistoryGarbageCleanupInfo(namespaceID.String(), workflowID, runID),
 			BranchToken:       branchToken,
 			Events:            events,
 			PrevTransactionID: prevTxnID,
@@ -248,7 +247,7 @@ func persistNonFirstWorkflowEvents(
 		return 0, nil // allow update workflow without events
 	}
 
-	namespaceID := workflowEvents.NamespaceID
+	namespaceID := namespace.ID(workflowEvents.NamespaceID)
 	execution := commonpb.WorkflowExecution{
 		WorkflowId: workflowEvents.WorkflowID,
 		RunId:      workflowEvents.RunID,
@@ -275,7 +274,7 @@ func persistNonFirstWorkflowEvents(
 
 func appendHistoryV2EventsWithRetry(
 	shard shard.Context,
-	namespaceID string,
+	namespaceID namespace.ID,
 	execution commonpb.WorkflowExecution,
 	request *persistence.AppendHistoryNodesRequest,
 ) (int64, error) {
@@ -314,6 +313,15 @@ func createWorkflowExecutionWithRetry(
 	)
 	switch err.(type) {
 	case nil:
+		if namespaceEntry, err := shard.GetNamespaceRegistry().GetNamespaceByID(
+			namespace.ID(request.NewWorkflowSnapshot.ExecutionInfo.NamespaceId),
+		); err == nil {
+			emitMutationMetrics(
+				shard,
+				namespaceEntry,
+				&resp.NewMutableStateStats,
+			)
+		}
 		return resp, nil
 	case *persistence.CurrentWorkflowConditionFailedError,
 		*persistence.WorkflowConditionFailedError,
@@ -334,6 +342,63 @@ func createWorkflowExecutionWithRetry(
 	}
 }
 
+func conflictResolveWorkflowExecutionWithRetry(
+	shard shard.Context,
+	request *persistence.ConflictResolveWorkflowExecutionRequest,
+) (*persistence.ConflictResolveWorkflowExecutionResponse, error) {
+
+	var resp *persistence.ConflictResolveWorkflowExecutionResponse
+	op := func() error {
+		var err error
+		resp, err = shard.ConflictResolveWorkflowExecution(request)
+		return err
+	}
+
+	err := backoff.Retry(
+		op,
+		PersistenceOperationRetryPolicy,
+		common.IsPersistenceTransientError,
+	)
+	switch err.(type) {
+	case nil:
+		if namespaceEntry, err := shard.GetNamespaceRegistry().GetNamespaceByID(
+			namespace.ID(request.ResetWorkflowSnapshot.ExecutionInfo.NamespaceId),
+		); err == nil {
+			emitMutationMetrics(
+				shard,
+				namespaceEntry,
+				&resp.ResetMutableStateStats,
+				resp.NewMutableStateStats,
+				resp.CurrentMutableStateStats,
+			)
+			emitCompletionMetrics(
+				shard,
+				namespaceEntry,
+				snapshotToCompletionMetric(&request.ResetWorkflowSnapshot),
+				snapshotToCompletionMetric(request.NewWorkflowSnapshot),
+				mutationToCompletionMetric(request.CurrentWorkflowMutation),
+			)
+		}
+		return resp, nil
+	case *persistence.CurrentWorkflowConditionFailedError,
+		*persistence.WorkflowConditionFailedError,
+		*persistence.ConditionFailedError:
+		// it is possible that workflow already exists and caller need to apply
+		// workflow ID reuse policy
+		return nil, err
+	default:
+		shard.GetLogger().Error(
+			"Persistent store operation Failure",
+			tag.WorkflowNamespaceID(request.ResetWorkflowSnapshot.ExecutionInfo.NamespaceId),
+			tag.WorkflowID(request.ResetWorkflowSnapshot.ExecutionInfo.WorkflowId),
+			tag.WorkflowRunID(request.ResetWorkflowSnapshot.ExecutionState.RunId),
+			tag.StoreOperationConflictResolveWorkflowExecution,
+			tag.Error(err),
+		)
+		return nil, err
+	}
+}
+
 func getWorkflowExecutionWithRetry(
 	shard shard.Context,
 	request *persistence.GetWorkflowExecutionRequest,
@@ -354,6 +419,15 @@ func getWorkflowExecutionWithRetry(
 	)
 	switch err.(type) {
 	case nil:
+		if namespaceEntry, err := shard.GetNamespaceRegistry().GetNamespaceByID(
+			namespace.ID(resp.State.ExecutionInfo.NamespaceId),
+		); err == nil {
+			emitGetMetrics(
+				shard,
+				namespaceEntry,
+				&resp.MutableStateStats,
+			)
+		}
 		return resp, nil
 	case *serviceerror.NotFound:
 		// it is possible that workflow does not exists
@@ -362,8 +436,8 @@ func getWorkflowExecutionWithRetry(
 		shard.GetLogger().Error(
 			"Persistent fetch operation Failure",
 			tag.WorkflowNamespaceID(request.NamespaceID),
-			tag.WorkflowID(request.Execution.WorkflowId),
-			tag.WorkflowRunID(request.Execution.RunId),
+			tag.WorkflowID(request.WorkflowID),
+			tag.WorkflowRunID(request.RunID),
 			tag.StoreOperationGetWorkflowExecution,
 			tag.Error(err),
 		)
@@ -389,17 +463,20 @@ func updateWorkflowExecutionWithRetry(
 	)
 	switch err.(type) {
 	case nil:
-		// TODO @wxing1292
-		//  temporarily move the emission of per update mutable state metrics
-		//  to here, long term story, this emission of metrics should have a
-		//  dedicated layer
-		if namespaceEntry, err := shard.GetNamespaceCache().GetNamespaceByID(
-			request.UpdateWorkflowMutation.ExecutionInfo.NamespaceId,
+		if namespaceEntry, err := shard.GetNamespaceRegistry().GetNamespaceByID(
+			namespace.ID(request.UpdateWorkflowMutation.ExecutionInfo.NamespaceId),
 		); err == nil {
-			emitSessionUpdateStats(
-				shard.GetMetricsClient(),
-				namespaceEntry.GetInfo().Name,
-				resp.MutableStateUpdateSessionStats,
+			emitMutationMetrics(
+				shard,
+				namespaceEntry,
+				&resp.UpdateMutableStateStats,
+				resp.NewMutableStateStats,
+			)
+			emitCompletionMetrics(
+				shard,
+				namespaceEntry,
+				mutationToCompletionMetric(&request.UpdateWorkflowMutation),
+				snapshotToCompletionMetric(request.NewWorkflowSnapshot),
 			)
 		}
 		return resp, nil
@@ -455,10 +532,10 @@ func NotifyWorkflowMutationTasks(
 
 func notifyTasks(
 	engine shard.Engine,
-	transferTasks []persistence.Task,
-	timerTasks []persistence.Task,
-	replicationTasks []persistence.Task,
-	visibilityTasks []persistence.Task,
+	transferTasks []tasks.Task,
+	timerTasks []tasks.Task,
+	replicationTasks []tasks.Task,
+	visibilityTasks []tasks.Task,
 ) {
 	engine.NotifyNewTransferTasks(transferTasks)
 	engine.NotifyNewTimerTasks(timerTasks)
@@ -553,3 +630,82 @@ func NotifyNewHistoryMutationEvent(
 	))
 	return nil
 }
+
+func emitMutationMetrics(
+	shard shard.Context,
+	namespace *namespace.Namespace,
+	stats ...*persistence.MutableStateStatistics,
+) {
+	metricsClient := shard.GetMetricsClient()
+	namespaceName := namespace.Name()
+	for _, stat := range stats {
+		emitMutableStateStatus(
+			metricsClient.Scope(metrics.SessionSizeStatsScope, metrics.NamespaceTag(namespaceName.String())),
+			metricsClient.Scope(metrics.SessionCountStatsScope, metrics.NamespaceTag(namespaceName.String())),
+			stat,
+		)
+	}
+}
+
+func emitGetMetrics(
+	shard shard.Context,
+	namespace *namespace.Namespace,
+	stats ...*persistence.MutableStateStatistics,
+) {
+	metricsClient := shard.GetMetricsClient()
+	namespaceName := namespace.Name()
+	for _, stat := range stats {
+		emitMutableStateStatus(
+			metricsClient.Scope(metrics.ExecutionSizeStatsScope, metrics.NamespaceTag(namespaceName.String())),
+			metricsClient.Scope(metrics.ExecutionCountStatsScope, metrics.NamespaceTag(namespaceName.String())),
+			stat,
+		)
+	}
+}
+
+func snapshotToCompletionMetric(
+	workflowSnapshot *persistence.WorkflowSnapshot,
+) completionMetric {
+	if workflowSnapshot == nil {
+		return completionMetric{initialized: false}
+	}
+	return completionMetric{
+		initialized: true,
+		taskQueue:   workflowSnapshot.ExecutionInfo.TaskQueue,
+		status:      workflowSnapshot.ExecutionState.Status,
+	}
+}
+
+func mutationToCompletionMetric(
+	workflowMutation *persistence.WorkflowMutation,
+) completionMetric {
+	if workflowMutation == nil {
+		return completionMetric{initialized: false}
+	}
+	return completionMetric{
+		initialized: true,
+		taskQueue:   workflowMutation.ExecutionInfo.TaskQueue,
+		status:      workflowMutation.ExecutionState.Status,
+	}
+}
+
+func emitCompletionMetrics(
+	shard shard.Context,
+	namespace *namespace.Namespace,
+	completionMetrics ...completionMetric,
+) {
+	metricsClient := shard.GetMetricsClient()
+	namespaceName := namespace.Name()
+
+	for _, completionMetric := range completionMetrics {
+		if !completionMetric.initialized {
+			continue
+		}
+		emitWorkflowCompletionStats(
+			metricsClient,
+			namespaceName,
+			completionMetric.taskQueue,
+			completionMetric.status,
+		)
+	}
+}
