@@ -25,6 +25,7 @@
 package replicator
 
 import (
+	"sync"
 	"sync/atomic"
 
 	"go.temporal.io/server/client"
@@ -45,12 +46,14 @@ type (
 		clusterMetadata                  cluster.Metadata
 		namespaceReplicationTaskExecutor namespace.ReplicationTaskExecutor
 		clientBean                       client.Bean
-		namespaceProcessors              []*namespaceReplicationMessageProcessor
 		logger                           log.Logger
 		metricsClient                    metrics.Client
 		hostInfo                         *membership.HostInfo
 		serviceResolver                  membership.ServiceResolver
 		namespaceReplicationQueue        persistence.NamespaceReplicationQueue
+
+		namespaceProcessorsLock sync.Mutex
+		namespaceProcessors     map[string]*namespaceReplicationMessageProcessor
 	}
 
 	// Config contains all the replication config for worker
@@ -70,37 +73,15 @@ func NewReplicator(
 	namespaceReplicationTaskExecutor namespace.ReplicationTaskExecutor,
 ) *Replicator {
 
-	logger = log.With(logger, tag.ComponentReplicator)
-	var namespaceReplicationMessageProcessors []*namespaceReplicationMessageProcessor
-	currentClusterName := clusterMetadata.GetCurrentClusterName()
-	for targetClusterName, info := range clusterMetadata.GetAllClusterInfo() {
-		if !info.Enabled {
-			continue
-		}
-
-		if targetClusterName != currentClusterName {
-			namespaceReplicationMessageProcessors = append(namespaceReplicationMessageProcessors, newNamespaceReplicationMessageProcessor(
-				currentClusterName,
-				targetClusterName,
-				log.With(logger, tag.ComponentReplicationTaskProcessor, tag.SourceCluster(targetClusterName)),
-				clientBean.GetRemoteAdminClient(targetClusterName),
-				metricsClient,
-				namespaceReplicationTaskExecutor,
-				hostInfo,
-				serviceResolver,
-				namespaceReplicationQueue,
-			))
-		}
-	}
 	return &Replicator{
 		status:                           common.DaemonStatusInitialized,
 		hostInfo:                         hostInfo,
 		serviceResolver:                  serviceResolver,
 		clusterMetadata:                  clusterMetadata,
 		namespaceReplicationTaskExecutor: namespaceReplicationTaskExecutor,
-		namespaceProcessors:              namespaceReplicationMessageProcessors,
+		namespaceProcessors:              make(map[string]*namespaceReplicationMessageProcessor),
 		clientBean:                       clientBean,
-		logger:                           logger,
+		logger:                           log.With(logger, tag.ComponentReplicator),
 		metricsClient:                    metricsClient,
 		namespaceReplicationQueue:        namespaceReplicationQueue,
 	}
@@ -116,9 +97,7 @@ func (r *Replicator) Start() {
 		return
 	}
 
-	for _, namespaceProcessor := range r.namespaceProcessors {
-		namespaceProcessor.Start()
-	}
+	r.listenToClusterMetadataChange()
 }
 
 // Stop is called to stop replicator
@@ -131,7 +110,49 @@ func (r *Replicator) Stop() {
 		return
 	}
 
+	currentClusterName := r.clusterMetadata.GetCurrentClusterName()
+	r.clusterMetadata.UnRegisterMetadataChangeCallback(currentClusterName)
+	r.namespaceProcessorsLock.Lock()
+	defer r.namespaceProcessorsLock.Unlock()
 	for _, namespaceProcessor := range r.namespaceProcessors {
 		namespaceProcessor.Stop()
 	}
 }
+
+func (r *Replicator) listenToClusterMetadataChange() {
+	currentClusterName := r.clusterMetadata.GetCurrentClusterName()
+	r.clusterMetadata.RegisterMetadataChangeCallback(
+		currentClusterName,
+		func(
+			oldClusterMetadata map[string]*cluster.ClusterInformation,
+			newClusterMetadata map[string]*cluster.ClusterInformation,
+		) {
+			r.namespaceProcessorsLock.Lock()
+			defer r.namespaceProcessorsLock.Unlock()
+			for clusterName := range newClusterMetadata {
+				if clusterName == currentClusterName {
+					continue
+				}
+				if processor, ok := r.namespaceProcessors[clusterName]; ok {
+					processor.Stop()
+					delete(r.namespaceProcessors, clusterName)
+				}
+				if clusterInfo := newClusterMetadata[clusterName]; clusterInfo != nil && clusterInfo.Enabled {
+					processor := newNamespaceReplicationMessageProcessor(
+						currentClusterName,
+						clusterName,
+						log.With(r.logger, tag.ComponentReplicationTaskProcessor, tag.SourceCluster(clusterName)),
+						r.clientBean.GetRemoteAdminClient(clusterName),
+						r.metricsClient,
+						r.namespaceReplicationTaskExecutor,
+						r.hostInfo,
+						r.serviceResolver,
+						r.namespaceReplicationQueue,
+					)
+					processor.Start()
+					r.namespaceProcessors[clusterName] = processor
+				}
+			}
+		},
+	)
+}
